//
// JavaScript Library "bigint.js v0.5" beta
// Completely Free
//
// core.js
//
// LIMITATION
// _getStringZeros: zeros must be <= 16777216
// Bigint.fromFloat: Power must be <= 16777216
// in log.js, POWER must be < 7.807...e+307
//////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////
//
// header
//
//////////////////////////////////////////////////////////////////////////////////////

var __BIGINT_RADIX = 10;

var __BIGINT_POWER = 7;

var __BIGINT_UNIT = Math.pow( __BIGINT_RADIX, __BIGINT_POWER );
var __BIGINT_UNIT_MINUS_1 = __BIGINT_UNIT - 1;
var __BIGINT_HALF_UNIT = __BIGINT_UNIT / 2;

var __BIGINT_JS_INT_MAX = Math.pow(2, 53);// 90 0719925 4740992

var __BIGINT_JS_INT_MAX_SQRT = Math.floor( Math.sqrt( __BIGINT_JS_INT_MAX ) );

var __BIGINT_JS_FLOAT_MAX = Number.MAX_VALUE; // 1.7976931348623157e+308;
var __BIGINT_JS_COMPATIBLE_MAX_CELLS = Math.floor( 308 / __BIGINT_POWER );

var __BIGINT_OVERFLOW_FREE_CELLS = Math.floor( __BIGINT_JS_INT_MAX / (Math.pow( ( __BIGINT_UNIT - 1 ), 2 )) );
var __BIGINT_OVERFLOW_PROTECTION_THRESHOLD_MUL = __BIGINT_JS_INT_MAX - Math.pow( ( __BIGINT_UNIT - 1 ), 2 )
	- ( __BIGINT_UNIT - 1 ) - Math.floor ( __BIGINT_JS_INT_MAX / ( __BIGINT_UNIT * __BIGINT_UNIT ) );
var __BIGINT_OVERFLOW_PROTECTION_THRESHOLD_SQ = __BIGINT_JS_INT_MAX - Math.pow( ( __BIGINT_UNIT - 1 ), 2 ) * 2
	- ( __BIGINT_UNIT - 1 ) - Math.floor ( __BIGINT_JS_INT_MAX / ( __BIGINT_UNIT * __BIGINT_UNIT ) );

var __BIGINT_BIGGER_POWER = 15;
var __BIGINT_BIGGER_UNIT = Math.pow( __BIGINT_RADIX, __BIGINT_BIGGER_POWER );
var __BIGINT_BIGGER_HALF_UNIT = __BIGINT_BIGGER_UNIT / 2;
var __BIGINT_2_16 = Math.pow( 2, 16 );
var __BIGINT_2_32 = Math.pow( 2, 32 );
var __BIGINT_10_12 = Math.pow( 10, 12 );
var __BIGINT_10_13 = Math.pow( 10, 13 );
var __BIGINT_10_14 = Math.pow( 10, 14 );
var __BIGINT_10_15 = Math.pow( 10, 15 );

var __BIGINT_MOZ = ( typeof navigator === "object" && typeof navigator.userAgent === "string" && navigator.userAgent.indexOf( "Gecko" ) > -1 );

var undefined = void 0;

var __BIGINT_INVALID_NUMBER = void 0;

//////////////////////////////////////////////////////////////////////////////////////
//
// constructor
//
//////////////////////////////////////////////////////////////////////////////////////


function Bigint( input ) {
	this.isBigint = true;
	this.table = [ 0 ];
	this.isNonNegative = true;
	if( !input ) return;

	var strInput;

	if( typeof input === "number" ) {
		if( input > 0 )
		{
			strInput = String( input );
		}
		else if( input > -1 )
		{
			return;
		}
		else
		{
			this.isNonNegative = false;
			strInput = String( Math.abs( input ) );
		}
		// if( strInput.match( /e/i ) ) return Bigint.fromFloat( input );
	} else {
		strInput = String( input ).replace( /\s/g , "" );
		var posStart;
		switch( strInput.charAt( 0 ) ) {
			case "-": this.isNonNegative = false; posStart = 1; break;
			case "+": posStart = 1; break;
			default: posStart = 0; break;
		}

		while( strInput.charAt( posStart ) === "0" ) { posStart++; }
		var len = strInput.length;
		var posEnd = posStart;
		for( ; posEnd < len; posEnd++ ) {
			if( isNaN( strInput.charAt( posEnd ) ) ) break;
		}
		if( posStart === posEnd ) { 
			if( strInput.charAt( 0 ) === "0" || posStart > 1 ) { this.isNonNegative = true; return; }
			else { Bigint._error( "constructor", "Bad input : " + input ); return; }
		}
		strInput = strInput.substring( posStart, posEnd );
	}

	var nLength = strInput.length;
	var nLastIndex = Math.ceil( nLength / __BIGINT_POWER ) - 1;
	var nLastItemLength = nLength % __BIGINT_POWER || __BIGINT_POWER;

	this.table[ nLastIndex ] = Number( strInput.substr( 0, nLastItemLength ) );
	var pos = nLastItemLength;
	for( var i = nLastIndex - 1; i > -1; i--, pos += __BIGINT_POWER ) {
		this.table[ i ] = Number( strInput.substr( pos , __BIGINT_POWER ) );
	}
}

//////////////////////////////////////////////////////////////////////////////////////
//
// Bigint.serio & Bigint.prototype.copy
//
//////////////////////////////////////////////////////////////////////////////////////


Bigint.serio = function( input ) {
	//if( input && input.isBigint ) return input;
	if( input.isBigint ) return input;
	return new Bigint( input );
}


Bigint.prototype.copy = function() {
	var table1 = this.table;
	var len = table1.length;
	var table2 = new Array( len );
	for( var i = 0; i < len; i++ ) {
		table2[ i ] = table1[ i ];
	}
	var bigNew = new Bigint();
	bigNew.isNonNegative = this.isNonNegative;
	bigNew.table = table2;
	return bigNew;
}

//////////////////////////////////////////////////////////////////////////////////////
//
// Bigint.parse
//
//////////////////////////////////////////////////////////////////////////////////////

Bigint.parse = function( input ) {
//	if( typeof input === "number" && isNaN( input ) || input === Infinity || input === -Infinity )
//		return Bigint._error( "parse" , "input is " + input );
	var strInput = String( input ).replace( /\s/g , "" );

	// Check the possibility of Hex first; otherwise we'll be confused by a string like "0x3e50"
	if( strInput.match( /^[\+\-]?0x/i ) ) return Bigint.fromHex( strInput );
	else if( strInput.match( /e[\+\-]?\d+$/i ) ) return Bigint.fromFloat( strInput );
	else return new Bigint( strInput );
}

Bigint.n = Bigint.number = Bigint.read = Bigint.load = Bigint.n = Bigint.parse;

Bigint._n = function( n ) {
	var ret = new Bigint();
	ret.table[ 0 ] = n;
	return ret;
}


//////////////////////////////////////////////////////////////////////////////////////
//
// Bigint.fromFloat
//
//////////////////////////////////////////////////////////////////////////////////////

Bigint.fromFloat = function( input, power ) {

	if( power ) input += "e" + power;

	var strInput = String( input ).replace( /\s/g , "" ).toLowerCase();
	if( !strInput.match( /e/ ) ) return new Bigint( strInput );

	var bNegative = false;

	var posStart;

	switch( strInput.charAt( 0 ) ) {
		case "+": posStart = 1; break;
		case "-": posStart = 1; bNagative = true; break;
		default: posStart = 0; break;
	}

	while( strInput.charAt( posStart ) === "0" ) { posStart++; }
	var posE = strInput.indexOf( "e" );

	var strNumber = strInput.substring( posStart , posE );

	if( !strNumber ) {
		if( strInput.charAt( 0 ) === "0" || posStart > 1 ) strNumber = "0";
		else return Bigint._error( "fromFloat", "Mantissa not recognized: " + input );
	}

	var strPower = strInput.substr( posE + 1 );
	var nPower = Number( strPower );

	if( strPower === "" || isNaN( nPower ) || nPower != Math.floor( nPower ) ) {
		return Bigint._error( "fromFloat" , "Scaling factor not found or non-integer: " + input );
	}
	var nMove = Math.abs( nPower );

	if( nMove > 16777216 ) {
		return Bigint._error( "fromFloat" , "Scaling factor too big: " + input );
	}

	var posP = strNumber.indexOf( "." );
	var strNumberHigh, strNumberLow;
	if( posP < 0 ) {
		strNumberHigh = strNumber;
		strNumberLow = "";
	} else {
		strNumberHigh = strNumber.substring( 0 , posP );
		strNumberLow = strNumber.substr( posP + 1 );
		if( strNumberLow === "" ) {
			return Bigint._error( "fromFloat" , "Illegal decimal point: " + input );
		}
	}

	if( strNumberHigh.match( /\D/ ) || strNumberLow.match( /\D/ ) ) {
		return Bigint._error( "fromFloat" , "Malformed mantissa: " + input );
	} else if( Number( strNumber ) === 0 ) {
		return new Bigint( 0 );
	}

 	if( nPower > 0 ) {
		var nLowLength = strNumberLow.length;
		if( nLowLength >= nMove ) strNumberLow = strNumberLow.substr( 0 , nMove );
		else return Bigint._fromString( strNumberHigh + strNumberLow , nMove - nLowLength , bNegative );
	} else {
		var nHighLength = strNumberHigh.length;
		if( nHighLength > nMove ) {
			strNumberHigh = strNumberHigh.substr( 0 , nHighLength - nMove );
			strNumberLow = "";
		} else return new Bigint( 0 );
	}

	var strSign = ( bNegative )? "-" : "";
	return new Bigint( strSign + strNumberHigh + strNumberLow );
}

Bigint._fromFloat = function( strInput ) { // Must be non-negative
	var pos1 = strInput.indexOf(".");
	var pos2 = strInput.indexOf("e");
	var nMove = ( pos2 < 0 )? 0 : Number( strInput.substr( pos2 + 1 ) ) ;

	if( pos1 < 0 ) {
		var strNumberHigh = strInput.substring( 0 , pos2 );
		var strNumberLow = "";
	} else {
		strNumberHigh = strInput.substring( 0 , pos1 );
		strNumberLow = strInput.substring( pos1 + 1 , pos2 );
	}
	var nLowLength = strNumberLow.length;
	if( nLowLength < nMove ) return Bigint._fromString( strNumberHigh + strNumberLow, nMove - nLowLength, false );
	else return new Bigint( strNumberHigh + strNumberLow.substr( 0 , nMove ) );
}


//////////////////////////////////////////////////////////////////////////////////////
//
// Bigint.fromHex
//
//////////////////////////////////////////////////////////////////////////////////////


Bigint.fromHex = function( input ) {
	if( !input ) return new Bigint( 0 );

	var strHex = String( input ).replace( /\s/ , "" );

	var bigResult = new Bigint( 0 );

	var posStart;

	switch( strHex.charAt( 0 ) ) {
		case "+": posStart = 1; break;
		case "-": posStart = 1; bigResult.isNonNegative = false; break;
		default: posStart = 0; break;
	}

	if( strHex.substr( posStart , 2 ).toLowerCase() === "0x" ) posStart += 2;

	strHex = strHex.substr( posStart );

	if( strHex.match( /[^0-9A-Fa-f]/ ) ) return Bigint._error( "fromHex" , "Bad input: " + input );

	var nPadding = 0;
	while( strHex.charAt( nPadding ) === "0" ) { nPadding++; }
	if( nPadding ) strHex = strHex.substr( nPadding );

	if( !strHex ) return new Bigint( 0 );

	var bigScalingFactor = new Bigint( 1 );
	var UNIT = Bigint.pow( 2 , 48 );
	for( var pos = strHex.length; pos > 0 ; pos -= 12, bigScalingFactor = Bigint._mul( bigScalingFactor, UNIT ) ) {
		var n = Number( "0x" + strHex.substring( pos - 12, pos ) );
		var bigTmp = Bigint._mul( new Bigint( n ), bigScalingFactor );
		bigResult = Bigint._add( bigResult, bigTmp );
	}
	return bigResult;
}

//////////////////////////////////////////////////////////////////////////////////////
//
// Bigint._fromString
//
//////////////////////////////////////////////////////////////////////////////////////

Bigint._fromString = function( strHead, nTrailingZeros, bNegative ) {
	var bigNumber = new Bigint();

	if( Number( strHead ) === 0 ) return new Bigint( 0 );

	if( !nTrailingZeros ) nTrailingZeros = 0;

	var nLength = strHead.length + nTrailingZeros;
	var nLastIndex = Math.ceil( nLength / __BIGINT_POWER ) - 1;
	var table = new Array( nLastIndex + 1 );
	var nLastItemLength = nLength % __BIGINT_POWER || __BIGINT_POWER;

	strHead += Bigint._getStringZeros( __BIGINT_POWER );
	table[ nLastIndex ] = Number( strHead.substr( 0, nLastItemLength ) );

	var pos = nLastItemLength;
	var i = nLastIndex - 1, end = strHead.length - __BIGINT_POWER;
	for( ; pos < end; i--, pos += __BIGINT_POWER ) {
		table[ i ] = Number( strHead.substr( pos , __BIGINT_POWER ) );
	}

	for( ; i > -1; i-- ) { table[ i ] = 0; }
	bigNumber.table = table;
	if( bNegative && !bigNumber.isZero() ) bigNumber.isNonNegative = false;
	return bigNumber;
}

//////////////////////////////////////////////////////////////////////////////////////
//
// Bigint._fromTable
//
// Data in table (possibly with Overflow) to Bigint
// Used from Bigint._mul and Bigint._square
// Input table must have a correct length; Otherwise the result will be unexpected
// Can't be negative
//
// WARNING: By reference, the input table will be substantially altered.
// Re-altering the input table anywhere else will mess up the result.
//////////////////////////////////////////////////////////////////////////////////////


Bigint._fromTable = function( table ) {
	var nTableLength = table.length;
	var n, nOverflow, nHigh, nSuperHigh;

	for( var i = 0; i < nTableLength; i++ ) {
		n = table[ i ];
/* ### */
///		if( n > __BIGINT_JS_INT_MAX ) return Bigint._error( "_fromTable",   "Lossy data: " + n + " at index " + i );

		if( n >= __BIGINT_UNIT ) {
			table[ i ] = n % __BIGINT_UNIT;
			nOverflow = ( n - table[ i ] ) / __BIGINT_UNIT;
			if( nOverflow >= __BIGINT_UNIT ) {
				nHigh = nOverflow %  __BIGINT_UNIT;
				nSuperHigh = ( nOverflow - nHigh ) / __BIGINT_UNIT;
				table[ i + 1 ] += nHigh;
				table[ i + 2 ] += nSuperHigh;
			} else {
				table[ i + 1 ] += nOverflow;
			}
		}
	}
	if( table[ table.length - 1 ] === 0 ) table.length--;
	var bigResult = new Bigint();
	bigResult.table = table;
	return bigResult;
}

// Single Overflows Only. (No nSuperHigh handling)
// The table length will be automatically increased by 1 if, and only if, needed.
Bigint._normalizeTable = function( table ) {
	var nTableLength = table.length;
	table[ nTableLength ] = 0;
	var n, nOverflow, nHigh, nSuperHigh;
	for( var i = 0; i < nTableLength; i++ ) {
		n = table[ i ];
		if( n >= __BIGINT_UNIT ) {
			table[ i ] = n % __BIGINT_UNIT;
			nOverflow = ( n - table[ i ] ) / __BIGINT_UNIT;
			table[ i + 1 ] += nOverflow;
		}
	}
	if( !table[ nTableLength ] ) table.length--;
}


//////////////////////////////////////////////////////////////////////////////////////
//
// compare etc.
//
//////////////////////////////////////////////////////////////////////////////////////

Bigint.prototype.comparedTo = function( compared ) {
	return this._comparedTo( Bigint.serio( compared ) );
}


Bigint.prototype._comparedTo = function( bigNumber ) {
	if( this.isNonNegative ) {
		if( bigNumber.isNonNegative ) return Bigint._tablecmp( this.table, bigNumber.table );
		else return 1;
	} else {
		if( bigNumber.isNonNegative ) return -1;
		else return Bigint._tablecmp( bigNumber.table, this.table );
	}
}


Bigint._tablecmp = function( table1, table2 ) {
	var len1 = table1.length;
	var len2 = table2.length;

	if( len1 === len2 ) {
		for( var i = len1 - 1; i > -1; i-- ) {
			if( table1[ i ] === table2[ i ] ) continue;
			else return table1[ i ] - table2[ i ];
		}
	} else return len1 - len2;

	return 0;
}



Bigint._abscmp = function( bigNumber1, bigNumber2 ) {
	return Bigint._tablecmp( bigNumber1.table, bigNumber2.table );
}



Bigint._strcmp = function( strNumber1, strNumber2 ) {
	var nLength1 = strNumber1.length;
	var nLength2 = strNumber2.length;

	if( nLength1 === nLength2 ) {
		for( var i = 0; i < nLength1; i += 15 ) {
			var a = Number( strNumber1.substr( i, 15 ) );
			var b = Number( strNumber2.substr( i, 15 ) );
			if( a !== b ) return a - b;
		}
		return 0;
	} else {
		for( var pos1 = 0; strNumber1.charAt( pos1 ) === "0"; pos1++ ) { nLength1--; }
		for( var pos2 = 0; strNumber2.charAt( pos2 ) === "0"; pos2++ ) { nLength2--; }
		if( nLength1 === nLength2 ) {
			for( ; pos1 < nLength1; pos1 += 15, pos2 += 15 ) {
				a = Number( strNumber1.substr( pos1, 15 ) );
				b = Number( strNumber2.substr( pos2, 15 ) );
				if( a !== b ) return a - b;
			}
			return 0;
		} else {
			return nLength1 - nLength2;
		}
	}
}

Bigint._strcmp2 = function( strNumber1, strNumber2 ) { // strNumber1 should be longer
//	var len1 = strNumber1.length;
	var len2 = strNumber2.length;
//	if( len1 < len2 ) return -1;
	strNumber1 = strNumber1.substr( 0 , len2 );

	for( var pos = 0; pos < len2; pos += 15 ) {
		var a = strNumber1.substr( pos, 15 );
		var b = strNumber2.substr( pos, 15 );
		if( a !== b ) return a - b;
	}
	return 0;
}

Bigint.prototype.isZero = function() {
	return ( this.table.length === 1 && !this.table[ 0 ] );
}

Bigint.prototype.isOdd = function() {
	return Boolean( this.table[ 0 ] & 1 );
}
Bigint.prototype.isEven = function() {
	return ( !( this.table[ 0 ] & 1 ) );
}

Bigint.prototype.getLength = function() {
	var len_1 = this.table.length - 1;
	return len_1 * __BIGINT_POWER + String( this.table[ len_1 ] ).length;
}

Bigint.prototype.getWidth = Bigint.prototype.getDigits = Bigint.prototype.getLength;

Bigint.prototype.isEqualTo = function( compared ) {
	return this._isEqualTo( Bigint.serio( compared ) );
}

Bigint.prototype._isEqualTo = function( bigNumber ) {
	var table1 = bigNumber.table, table2 = this.table;
	var len = table1.length;
	if( len !== table2.length ) return false;
	for( var i = 0; i < len; i++ ) {
		if( table1[ i ] === table2[ i ] ) continue;
		else return false;
	}
	return true;
}

Bigint.prototype.isGreaterThan = function( compared )
{
	return this._isGreaterThan( Bigint.serio( compared ) );
}

Bigint.prototype._isGreaterThan = function( bigNumber )
{
	if( this.isNonNegative ) {
		if( bigNumber.isNonNegative ) return ( Bigint._tablecmp( this.table, bigNumber.table ) > 0 );
		else return false;
	} else {
		if( bigNumber.isNonNegative ) return false;
		else return ( Bigint._tablecmp( bigNumber.table, this.table ) > 0 );
	}
}


Bigint._shift = function( bigNumber, n ) {
	var strNumber = bigNumber.toDecimalString();
	if( n > 0 ) strNumber += Bigint._getStringZeros( n );
	else strNumber = strNumber.substring( 0 , strNumber.length + n );
	return new Bigint( strNumber );
}

Bigint.parseNumber = function( input ) {

	if( input === void 0 ) return void 0;

	if( input.isBigint ) return input.toNumber();
	var n = Number( input );
	if( isNaN( n ) || n === Infinity || n === -Infinity ) return __BIGINT_INVALID_NUMBER;
	else return n;
}

