// factorial.js
//////////////////////////////////////////////////////////////////////////////////////

Bigint.factorial = function( input ) {
	if( input === undefined || input === "" ) {
		return Bigint._error( "factorial", "input void" );
	}
	var smallNumber = ( input.isBigint )? input.toNumber() : Number( input );
	if( isNaN( smallNumber ) || Math.floor( smallNumber ) !== smallNumber || smallNumber < 0 ) {
		return Bigint._error( "factorial", "bad input: " + input );
	} else if( smallNumber < 2 ) {
		return new Bigint( 1 );
	}
	if( smallNumber <= 900719 ) {
		return new Bigint( Bigint._factorial( smallNumber ) );
	}
	return Bigint._error( "factorial" , "input too big: " + input );
}

Bigint._factorial = function( nSmallNumber ) {
//_debug("Called for n = " + nSmallNumber);
	var nCalculationParam, N;
	if( nSmallNumber > 90071 ) { nCalculationParam = 10; N = 90071; }
	else if( nSmallNumber > 9007 ) { nCalculationParam = 11; N = 9007; }
	else if( nSmallNumber > 900 ) { nCalculationParam = 12; N = 900; }
	else if( nSmallNumber > 90 )  { nCalculationParam = 13; N = 1; }
	else { nCalculationParam = 14; N = 1; }

	var UNIT = Math.pow( 10, nCalculationParam );
	var PADDING = Bigint._padding[ nCalculationParam ];

	if( nSmallNumber > 900 ) {
//_debug("Calling...");
		var strSmallerFactorial = Bigint._factorial( N );

//_debug("Now Loading...");
		var len = strSmallerFactorial.length;
		var nTableSize = Math.ceil( len / nCalculationParam );
		var nLastItemLength = len % nCalculationParam || nCalculationParam;
		var myTable = new Array( nTableSize );
		myTable[ nTableSize - 1 ] = Number( strSmallerFactorial.substr( 0 , nLastItemLength ) );
		for( var i = nTableSize - 2, pos = nLastItemLength; i > -1; i--, pos += nCalculationParam ) {
			myTable[ i ] = Number( strSmallerFactorial.substr( pos, nCalculationParam ) );	
		}
		strSmallerFactorial = null;
		var nFirstNonzeroItemIndex = 0;
		for( i = 0; myTable[ i ] === 0; i++ ) nFirstNonzeroItemIndex++;
	} else {
		myTable = [ 1 ];
		nFirstNonzeroItemIndex = 0;
	}
//	var dummy = ( N + 1 ) % 1000;
	for( var n = N + 1; n <= nSmallNumber; n++/*, dummy++*/ ) {
		nTableSize = myTable.length;
/*
		if( dummy === 1000 ) {
			defaultStatus = ">>> " + n + "..."
			_timing(n + " (" + nCalculationParam+")" );
			dummy = 0;
		}
*/
		var nStartIndex = nFirstNonzeroItemIndex + 1;
		var nOverflow = 0;
		if( myTable[ nFirstNonzeroItemIndex ] ) {
			var nWork = myTable[ nFirstNonzeroItemIndex ] * n;
			myTable[ nFirstNonzeroItemIndex ] = nWork % UNIT;
			nOverflow =  ( nWork - myTable[ nFirstNonzeroItemIndex ] ) / UNIT;
		} else {
			nFirstNonzeroItemIndex++;
		}

		for( i = nStartIndex; i < nTableSize; i++ ) {
			nWork = nOverflow + myTable[ i ] * n;
			myTable[ i ] = nWork % UNIT;
			nOverflow =  ( nWork - myTable[ i ] ) / UNIT;
		}
		if( nOverflow ) myTable[ nTableSize ] = nOverflow;
	}
	var nLastIndex = myTable.length - 1;
	var strResult = String( myTable[ nLastIndex ] );
	for( i = nLastIndex - 1; i > -1; i-- ) { strResult += PADDING( myTable[ i ] ); }
	myTable = null;

//_timing("END for " + nSmallNumber);
	return strResult ;
}

Bigint.stirling = function( input, bReturnShortString ) {
	var n = ( input.isBigint )? input.toNumber() : Number( input ) ;

	if( isNaN( n ) || n < 0 || n !== Math.floor( n ) ) return Bigint._error( "sterling" , "Bad input: " + input );

	if( n > 100000 ) bReturnShortString = true;

	var ln = n * ( Math.log( n ) - 1 ) + Math.log( 2 * Math.PI * n ) / 2;
	if( ln === Infinity ) return Bigint._error( "stirling", "Too big input: " + input );

	var log10 = ln * Math.LOG10E;
	var power = Math.floor( log10 );
	var f = log10 - power;
	if( String( power ).indexOf( "e" ) > -1 ) { 
		if( power < 1e+100 ) power = Bigint.fromFloat( power ).toDecimalString();
		else power = "(" + power + ")";
	}
	var strHead = String( Math.pow( 10, f ) ).substr( 0, 10 );

	return ( bReturnShortString )? strHead + "e+" + power : Bigint.fromFloat( strHead , power ) ;
}

Bigint.getFactorialTrailingZeros = function( input ) {
	var bigInput = Bigint.serio( input );

	if( !bigInput.isNonNegative ) return Bigint._error( "getFactorialTrailingZeros" , "Bad input: " + input );

	var bigResult = new Bigint(0);

	var nMaxPower = Math.floor( Math.log( bigInput.toNumber() ) / Math.log( 5 ) );

	for( var nPower = 1; nPower <= nMaxPower; nPower++ ) {
if(nPower%10===0)defaultStatus=nPower;
		var bigDivisor = Bigint.pow( 5, nPower );
		var bigTmp = Bigint._qr( bigInput, bigDivisor )[ 0 ];
		 bigResult = Bigint._add( bigResult, bigTmp );
	}
	return bigResult;
}

Bigint.getFactorialTrailingZeros10E = function( input, bReturnLosses ) {
	var n = ( input.isBigint )? input.toNumber() : Number( input ) ;

	if( isNaN( n ) || n < 0 || n !== Math.floor( n ) || n === Infinity )
		return Bigint._error( "getFactorialTrailingZeros10E", "Bad input: " + input );
	if( n < 3 ) {
		switch( n ) {
			case 0: return ( bReturnLosses )? 0.25 : 0 ;
			case 1: return ( bReturnLosses )? 0.5 : 2 ;
			case 2: return ( bReturnLosses )? 1 : 24 ;
		}
	}

	var bigS = Bigint.pow( 2, n - 2 );
	var bigSi = new Bigint( 0 );
	var bigBase2 = Bigint.pow( 2, n + 1 );
	var nLimit = - Math.ceil( n * Math.LN2 / ( Math.LN10 - Math.LN2 ) );

	for( var nShift = -1; nShift > nLimit; nShift-- ) {
		var bigTmp = Bigint._shift( bigBase2, nShift );
		bigSi = Bigint._add( bigSi, bigTmp );
		bigBase2.twice();
if(nShift%10===0) defaultStatus=(nShift + " / " + nLimit);
	}

	var bigLosses = Bigint._sub( bigS, bigSi );
	return ( bReturnLosses )? bigLosses.toNumber() : Bigint._sub( Bigint.fromFloat( "2.5" , n - 1 ), bigLosses ) ;
}


Bigint.getFactorial10E = function( input ) {
	function log10(a){
		return Math.log(a)*Math.LOG10E;
	}

	var n = ( input.isBigint )? input.toNumber() : Number( input ) ;

	if( isNaN( n ) || n < 0 || n !== Math.floor( n ) || n === Infinity )
		return Bigint._error( "Bigint.getFactorial10E", "Bad input: " + input );

	if( n === 0 ) return [ "1" , new Bigint( 1 )];

	var str1_log10e =
	"5 6570551809 6748172348 8710810833 9491770560 2994196333 4338855462 1683413535 0791129225 2707750506 6156825168 1293893255 2336962663 5832071284 1036093430 7789353371 8773414787 2913431329 6704066291 3034117331 1668836392 2615094857 1556513332 3135341391 4864438517 8765123465 6456564268 2746164377 7186043969 5135334763 3904460622 6438236765 6808328900 8588402105 0370064875 4206507364 2344530922 3289175808 4952008901 0325099896 7224623464 2972991267 1449048268 5593253020 4810048640 5911959576 0684811318 9159745534 5910202970 1367131712 3737585598 6542956453 8670793992 8739489597 1632874045 1537122921 3800100767 3251560097 6518284640 6544892052 4507447517 4221793207 7985906853 1835532618 9694395243 6427959111 6616790511 0034772825 0545866820 8582359752 5924942112 3213902890 0742452269 9539513439 5048438994 2014258659 7273247985 6075208202 9140952068 7147875066 5880267012 2773536114 6497739161 1837368353 6116446314 4982315397 0471360060 8366489352 4442959494 8681765701 1125117879 3564049761 8109735668 2288462617 7966373655 8352160285 3998141603 9069936826 6601386596 4864258212 8550285470"; 
	var UNIT = Math.pow( 10, 10 );
	var bigWork = Bigint.fromFloat( ( n - 1 ) + "." + str1_log10e , n + 10 );
	var L = 0.39908993417905747; // log10( Math.sqrt( 2*Math.PI ) )
	var f = Math.floor( L * UNIT + n * ( UNIT / 2 ) );
	var strWork = Bigint.add( bigWork, f ).toDecimalString();

	var pos = strWork.length - 10;
	var bigResult = new Bigint( strWork.substring( 0 , pos ) );
	bigResult.increment();
	
	var strHead;
	switch( n ) {
		case 0: strHead = "1"; break;
		case 1: strHead = "362880"; break;
		case 2: strHead = "933262"; break;
		case 3: strHead = "402387"; break;
		case 4: strHead = "284625"; break;
		default:
			if( n <= 996 ) {
				var tmpstr = String( Math.pow( 10 , "0." + strWork.substring( pos ) ) ).replace( /\./ , "" );
				strHead = tmpstr.substr( 0 , 6 );
			} else {
				strHead = "?";
			}
		 break;
	}

	return [ strHead , bigResult ];

}

/*
ln10= 2.3025850929 9404568401 7991454684 3642076011 0148862877
           2976033327 9009675726 0967735248 0235997205 0895982983
           4196778404 2286248633 4095254650 8280675666 6287369098
           7816894829 0720832555 4680843799 8948262331 9852839350
           5308965377 7326288461 6336622228 7698219886 7465436674
           7440424327 4365155048 9343149393 9147961940 4400222105
           1017141748 0036880840 1264708068 5567743216 2283552201
           1480466371 5659121373 4507478569 4768346361 6792101806
           4450706480 0027750268 4916746550 5868569356 7342067058
           1136429224 5544057589 2572420824 1314695689 0167589402
           5677631135 6919292033 3765871416 6023010570 3089634572
           0754403708 4746994016 8269282808 4811842893 1484852494
           8644871927 8096762712 7577539702 7668605952 4967166741
           8348570442 2507197965 0047149510 5049221477 6567636938
           6629769795 2211071826 4549734772 6624257094 2932258279
           8502585509 7852653832 0760672631 7164309505 9950878075
           2371033310 1197857547 3315414218 0842754386 3591778117
           0543098274 8238504564 8019095610 2992918243 1823752535
           7709750539 5651876975 1037497088 8692180205 1893395072
           3853920514 4634197265 2872869651 1086257149 2198849979

*/
	/*
var 
strLog10e = 
"4 3429448190 3251827651 1289189166 0508229439 7005803666 5661144537 8316586464 9208870774 7292249493 3843174831 8706106744 7663037336 4167928715 8963906569 2210646628 1226585212 7086568670 3295933708 6965882668 8331163607 7384905142 8443486667 6864658608 5135561482 1234876534 3543435731 7253835622 2813956030 4864665236 6095539377 3561763234 3191671099 1411597894 9629935124 5793492635 7655469077 6710824191 5047991098 9674900103 2775376535 7027008732 8550951731 4406746979 5189951359 4088040423 9315188681 0840254465 4089797029 8632868287 6262414401 3457043546 1329206007 1260510402 8367125954 8462877078 6199899232 6748439902 3481715359 3455107947 5492552482 5778206792 2014093146 8164467381 0305604756 3572040888 3383209488 9965227174 9454133179 1417640247 4075057887 6786097109 9257547730 0460486560 4951561005 7985741340 2726752014 3924791797 0859047931 2852124933 4119732987 7226463885 3502260838 8162631646 3883553685 5017684602 9528639939 1633510647 5557040505 1318234298 8874882120 6435950238 1890264331 7711537382 2033626344 1647839714 6001858396 0930063173 3398613403 5135741787 1449714530  (1001-digit)";
*/
