// io.js


Bigint.prototype.toString = function( input ) {
	if( input === undefined ) return this.toFancyString( false );
	else if( input === 10 ) return this.toDecimalString( true );
	else if( input === 2 ) return this.toBinaryString( true );
	else if( input < 2 ) return this.toFancyString( true );

	var nRadix = Math.floor( input );

	if( isNaN( nRadix ) || nRadix > 36 ) {
		return Bigint._error( "toString" , "illegal radix: " + input );
	}

	return this._toStringRadix( nRadix );
}

Bigint.prototype.toFancyString = function( bForceAllDigits ) {
	var table = this.table;
	var nLength = table.length;

	var strSign = ( this.isNonNegative )? "" : "- ";

	var strBuffer = String( table[ nLength - 1 ] );
	var nDigits = strBuffer.length + ( nLength - 1 ) * __BIGINT_POWER;

	var strResult = strSign + strBuffer;
	var first = nDigits % 10 || 10;

	if( nDigits < 200 || bForceAllDigits ) {
		for( var i = nLength - 2; i > -1; i-- ) { strBuffer += __BIGINT_FN_PADDING( table[ i ] ); }
		for( var pos = first; pos < nDigits; pos += 10 ) { strResult += " " + strBuffer.substr( pos, 10 ); }
	} else {
		var stop = nLength - 5;
		for( i = nLength - 2; i > stop; i-- ) { strBuffer += __BIGINT_FN_PADDING( table[ i ] ); }
		strResult += " " + strBuffer.substr( first, 10 ) + "...";
	}

	return strResult + " (" + nDigits + "-digit)";

}


Bigint.prototype.toFancyString = function( bForceAllDigits ) {
	var table = this.table;
	var nLength = table.length;

	var strResult = ( this.isNonNegative )? "" : "- ";

	var strBuffer = String( table[ nLength - 1 ] );
	var nDigits = strBuffer.length + ( nLength - 1 ) * __BIGINT_POWER;
	var first = nDigits % 10 || 10;

	if( nDigits < 101 || bForceAllDigits ) {
		for( var i = nLength - 2; i > -1; i-- ) { strBuffer += __BIGINT_FN_PADDING( table[ i ] ); }

		strResult += strBuffer.substr( 0 , first );

		for( var pos = first; pos < nDigits; pos += 10 ) { strResult += " " + strBuffer.substr( pos, 10 ); }
	} else {
		var stop = nLength - 5;
		for( i = nLength - 2; i > stop; i-- ) {
			strBuffer += __BIGINT_FN_PADDING( table[ i ] );
		}

		strResult += strBuffer.substr( 0 , first ) + " " + strBuffer.substr( first, 10 ) + "...";
	}

	return strResult + " (" + nDigits + "-digit)";
}




Bigint.prototype.toDecimalString = function( bWithSign ) {
	var lastIndex = this.table.length-1;
	var strResult = "";
	if( bWithSign && !this.isNonNegative ) strResult = "-";
	strResult += this.table[ lastIndex ];

	for( var i = lastIndex - 1; i > -1; i-- ) {
		strResult += __BIGINT_FN_PADDING( this.table[i] );
	}

	return strResult;
}



Bigint.prototype.toBinaryString = function( bWithSign ) {
	var bigWork = this.copy();
	var table = bigWork.table;
	var strResult = "";

	do {
		if( Bigint._tableHalf( table ) ) {
			strResult = "1" + strResult;
		} else {
			strResult = "0" + strResult;
		}
	} while( table.length > 1 || table[0] );

	if( this.isNonNegative || !bWithSign ) return strResult;
	else return "-" + strResult;
}


Bigint.prototype._toStringRadix =  function( nRadix ) {
	var strDividend = this.toDecimalString();
	var strResult = "";
	do {
		var strQR = Bigint._qr_typeN( strDividend, nRadix );
		strResult = Number( strQR[1] ).toString( nRadix ) + strResult;
		strDividend = strQR[0];
	} while( strDividend !== "0" )

	return ( this.isNonNegative )? strResult : "-" + strResult ;
}

Bigint.prototype.toNumber = function() {
	return Number( this.toDecimalString( true ) );
}






Bigint._debug = function(str1, str2) {
	var strMessage = "DEBUG: ";
	if( str2 === undefined ) {
		strMessage += str1;
	} else {
		strMessage += str1 + " : " + str2;
	}

	if( typeof document !== "undefined" &&  typeof document.__BIGINT_HTML_FORM !== "undefined"  ) {
		if( str1 === undefined && str2 === undefined ) document.__BIGINT_HTML_FORM.debug.value = "";
		else
		document.__BIGINT_HTML_FORM.debug.value += strMessage + "\n";
	} else {
		document.write( "<p>" + strMessage + "<\/p>" );
	}


}

if( typeof _debug === "undefined" ){ var _debug = Bigint._debug; }


Bigint._error = function ( strFunctionName, strDescription ) {
	var strMessage = "ERROR: " + strFunctionName + "(): " + strDescription;
	if( typeof document !== "undefined"
		&& typeof document.__BIGINT_HTML_FORM !== "undefined"
		&& typeof document.__BIGINT_HTML_FORM.output !== "undefined" ) {
			document.__BIGINT_HTML_FORM.output.value = strMessage;
	} else {
		alert( strMessage );
	}

	Bigint._debug( strMessage );

	// To make an exception on purpose
	var dummy = Bigint.no_such_function(1);

	return undefined;
}


Bigint._progress = function( str ) {
	if( typeof document !== "undefined"
		&& typeof document.__BIGINT_HTML_FORM !== "undefined"
		&& typeof document.__BIGINT_HTML_FORM.output !== "undefined" ) {

		var p = document.__BIGINT_HTML_FORM.output;
		if( str === undefined ) p.value = "";
		else {
			p.value += str;
			if( p.value.length % 80 === 79 ) p.value += "\n";
			if( typeof defaultStatus !== "undefined" ) {
				var pos = p.value.lastIndexOf("\n");
				defaultStatus = p.value.substr( pos + 1 );
			}
		}
	}
}
if( typeof _progress === "undefined" ) { var _progress = Bigint._progress; }

Bigint.output = function( str ) {
	if( typeof document !== "undefined"
		&& typeof document.__BIGINT_HTML_FORM !== "undefined"
		&& typeof document.__BIGINT_HTML_FORM.output !== "undefined" ) {

		if( str === void 0 )  document.__BIGINT_HTML_FORM.output.value = "";
		else document.__BIGINT_HTML_FORM.output.value += str;
	}
	else {
		Bigint._debug( "IO Error" , "Bigint.output" );
	}
}



Bigint._timing = function( argument ) {
	var objDate = new Date();
	var t = objDate.getTime();

	if( typeof _timing.t0 === "undefined" || argument === 0 ) {
		_timing.t0 = t;
		_timing.totalCost = 0;
		return true;
	} else {
		var cost = t - _timing.t0;
		_timing.t0 = t;
		_timing.totalCost += cost;

		if( argument === undefined ) {
			Bigint._debug( "* Cost: " + cost + " ms");
			return cost;
		} else if( argument === "total" ) {
			var total = _timing.totalCost;
			Bigint._debug( "** Total Cost: " + total/1000 + " seconds");
			_timing.totalCost = 0;
			_timing.t0 = undefined;
			return total;
		} else {
			Bigint._debug( "*Cost ( " + argument + " ): " + cost + " ms");
			return cost;
		}
	}
}

if( typeof _timing === "undefined" ) { var _timing = Bigint._timing; }


_timing(0);
_timing.totalCost = 0;

Bigint.init = function() {
	if( document && document.__BIGINT_HTML_FORM ) {
		var f = document.__BIGINT_HTML_FORM;
		if( f.output !== undefined ) f.output.value = "";
		if( f.debug !== undefined ) f.debug.value = "";
		else return;

		f.debug.value = "-----" + __BIGINT_VERSION + "-----\n" + navigator.userAgent + "\n";
		if( defaultStatus !== undefined ) defaultStatus = "";
		var d = new Date();
		var Y = d.getFullYear();
		var M = Bigint._padding2( d.getMonth() + 1 );
		var D = Bigint._padding2( d.getDate() );
		var h = Bigint._padding2( d.getHours() );
		var m = Bigint._padding2( d.getMinutes() );
		var s = Bigint._padding2( d.getSeconds() );
		var ms = d.getTime() % 1000;
		ms = Bigint._padding3( ms );
		var offs = d.getTimezoneOffset();
		var TZ_sign = ( offs > 0 )? "-" : "+" ;
		offs = Math.abs( offs );
		var TZ_m = Bigint._padding2( offs % 60 );
		var TZ_h = Bigint._padding2( ( offs - TZ_m ) / 60 );
		var TZ = TZ_sign + TZ_h + ":" + TZ_m;
		f.debug.value += "Current Time: "
		 +  Y + "-" + M + "-" + D + "T" + h + ":" + m + ":" + s + "." + ms + TZ + "\n";
	}
}
