// misc.js

	var __BIGINT_TABLE_KANJI =  // This file par se is originally in UTF-8
		[
			"", // null 10**0
			"\u4e07", // String.fromCharCode( 19975 ) , // "万" 10**4
			"\u5104", //String.fromCharCode( 20740 ) , // "億" 10**8
			"\u5146", // String.fromCharCode( 20806 ) , // "兆" 10**12
			"\u4eac", // String.fromCharCode( 20140 ) , // "京" 10**16
			"\u5793", // String.fromCharCode( 22419 ) , // "垓" 10**20
			"\u79ed", // String.fromCharCode( 31213 ) , // "秭" (U+79ED) 10**24
			"\u7a63", // String.fromCharCode( 31331 ) , // "穣" 10**28
			"\u6e9d", // String.fromCharCode( 28317 ) , // "溝" 10**32
			"\u6e97", // String.fromCharCode( 28567 ) , // "澗" 10**36
			"\u6b63", // String.fromCharCode( 27491 ) , // "正" 10**40
			"\u8fd9", // String.fromCharCode( 36617 ) , // "載" 10**44
			"\u6975", // String.fromCharCode( 26997 ) , // "極" 10**48
			"\u9052\u6cb3\u6c99", // String.fromCharCode( 24658,27827,27801 ) , // "恒河沙" 10**52
			"\u963f\u50e7\u7947", // String.fromCharCode( 38463,20711,31047 ) , // "阿僧祇" 10**56
			"\u90a3\u7531\u4ed6", // String.fromCharCode( 37027,30001,20182 ) , // "那由他" 10**60
			"\u4e0d\u53ef\u601d\u8b70", // String.fromCharCode( 19981,21487,24605,35696 ) , // "不可思議" 10**64
			"\u7121\u91cf\u5927\u6570", // String.fromCharCode( 28961,37327,22823,25968 ) // "無量大数" 10**68
	];

//////////////////////////////////////////////////////////////////////////////////////

Bigint.prototype.toKanji = function() {
	var strSign = this.isNonNegative? "" : "-";
	return strSign + Bigint._toKanji( this.toDecimalString(), false );
}

Bigint._toKanji = function( strNumber, bRecursive ) {
	var len = strNumber.length;
	var strHigh, strLow;
	if( !bRecursive && len <= 72 || len <=56 ) return Bigint.__toKanji( strNumber );
	else if( len <= 112 ) {
		strHigh = strNumber.substr( 0, len-56 );
		strLow = strNumber.substr( len-56 );
		return Bigint._toKanji( strHigh ) + "\u983b\u6ce2\u7f85" + Bigint._toKanji( strLow );//頻波羅
	} else if( len <= 224 ) {
		strHigh = strNumber.substr( 0, len-112 );
		strLow = strNumber.substr( len-112 );
		return Bigint._toKanji( strHigh, true ) + "\u77dc\u7faf\u7f85" + Bigint._toKanji( strLow, true );//矜羯羅
	} else if( len <= 448 ) {
		strHigh = strNumber.substr( 0, len-224 );
		strLow = strNumber.substr( len-224 );
		return Bigint._toKanji( strHigh, true ) + "\u963f\u4f3d\u7f85" + Bigint._toKanji( strLow, true );//阿伽羅
	} else if( len <= 896 ) {
		strHigh = strNumber.substr( 0, len-448 );
		strLow = strNumber.substr( len-448 );
		return Bigint._toKanji( strHigh, true ) + "\u6700\u52dd" + Bigint._toKanji( strLow, true );//最勝
	} else if( len <= 1792 ) {
		strHigh = strNumber.substr( 0, len-896 );
		strLow = strNumber.substr( len-896 );
		return Bigint._toKanji( strHigh, true ) + "\u6469\u5a46\u7f85" + Bigint._toKanji( strLow, true );//摩婆羅
	} else if( len <= 3584 ) {
		strHigh = strNumber.substr( 0, len-1792 );
		strLow = strNumber.substr( len-1792 );
		return Bigint._toKanji( strHigh, true ) + "\u963f\u5a46\u7f85" + Bigint._toKanji( strLow, true );//阿婆羅
	} else if( len <= 7168 ) {
		strHigh = strNumber.substr( 0, len-3584 );
		strLow = strNumber.substr( len-3584 );
		return Bigint._toKanji( strHigh, true ) + "\u591a\u5a46\u7f85" + Bigint._toKanji( strLow, true );//多婆羅
	} else if( len <= 14336 ) {
		strHigh = strNumber.substr( 0, len-7168 );
		strLow = strNumber.substr( len-7168 );
		return Bigint._toKanji( strHigh, true ) + "\u754c\u5206" + Bigint._toKanji( strLow, true );// 界分 
	} else if( len <= 28672 ) {
		strHigh = strNumber.substr( 0, len-14336 );
		strLow = strNumber.substr( len-14336 );
		return Bigint._toKanji( strHigh, true ) + "\u666e\u6469" + Bigint._toKanji( strLow, true );// 普摩 
	} else if( len <= 57344 ) {
		strHigh = strNumber.substr( 0, len-28672 );
		strLow = strNumber.substr( len-28672 );
		return Bigint._toKanji( strHigh, true ) + "\u79b0\u6469" + Bigint._toKanji( strLow, true );// 禰摩 
	}

	return "toKanji: too big to parse: " + len + "-digit";

}

Bigint.__toKanji = function( strNumber ) {

	var nFirstItemLength = strNumber.length % 4;
	if( nFirstItemLength === 0 ) nFirstItemLength = 4;

	var nPower = strNumber.length - 1;
	var nUnitIndex = Math.floor( nPower / 4 );

	var strResult = strNumber.substring( 0 , nFirstItemLength ) + __BIGINT_TABLE_KANJI[ nUnitIndex-- ];

	for( var pos = nFirstItemLength; pos+4 <= strNumber.length; pos += 4, nUnitIndex-- ) {
		var tmpstr = strNumber.substring( pos, pos+4 );
		if( tmpstr !== "0000" ) strResult += strNumber.substring( pos, pos+4 ) + __BIGINT_TABLE_KANJI[ nUnitIndex ] ;
	}
	return strResult;
}


Bigint.unicode = new Object();


Bigint.unicode.UTF8ToString = function( pUTF8 ) {
	
	var strResult = "";
	
	for( var pos=0; pos < pUTF8.length; pos++ ) {
		var pUTF8Char = new Array();
		var nHead = pUTF8[ pos ];
		if( nHead <= 127 ) {
			pUTF8Char[0] = nHead;
		} else if( nHead < 224 ) {
			pUTF8Char[0] = nHead;
			pUTF8Char[1] = pUTF8[ ++pos ];
		} else {
			pUTF8Char[0] = nHead;
			pUTF8Char[1] = pUTF8[ ++pos ];
			pUTF8Char[2] = pUTF8[ ++pos ];
		}

		var nUTF16Code = PigPGP.unicode.UTF8To16( pUTF8Char );
		strResult += String.fromCharCode( nUTF16Code );
	}

	return strResult;
}


Bigint.unicode.UTF16To8 = function( nUTF16Code ) {
	if( typeof nUTF16Code !== "number" ) {
		return Bigint._error( "Bigint.uniocde.UTF16To8" , "Input is " + typeof nUTF16Code + ": " + nUTF16Code );
	}
	
	var pUTF8Code = new Array();
	var q, r;
	if( nUTF16Code <= 0x007F ) pUTF8Code[0] = nUTF16Code;
	else if( nUTF16Code <= 0x07FF ) {
		r = nUTF16Code % 64;
		q = ( nUTF16Code - r ) / 64;
		pUTF8Code[1] = 128 + r;
		pUTF8Code[0] = 192 + q;
	} else if( nUTF16Code >= 0xD800 && nUTF16Code <= 0xDFFF ) {
		return Bigint._error( "Bigint.uniocde.UTF16To8" ,
			"Surrogate pairs not supported: 0x" +  nUTF16Code.toString(16) );
	} else if( nUTF16Code <= 0xFFFF ) {
		r = nUTF16Code % 64;
		q = ( nUTF16Code - r ) / 64;
		pUTF8Code[2] = 128 + r;
		r = q % 64;
		q = ( q - r ) / 64;
		pUTF8Code[1] = 128 + r;
		pUTF8Code[0] = 224 + q;
	} else {
		return Bigint._error( "Bigint.uniocde.UTF16To8" ,
			"Input not in BMP: 0x" +  nUTF16Code.toString(16) );
	}

	return pUTF8Code;
}

Bigint.unicode.UTF8To16 = function( pUTF8Code ) {
	if( typeof pUTF8Code !== "object" || typeof pUTF8Code[0] === "undefined" ) {
		return Bigint._error( "Bigint.uniocde.UTF8To16" ,
		"Input is " + typeof nUTF8Code + ": " + nUTF8Code );
	}
	var n0, n1, n2;

	switch( pUTF8Code.length ) {
		case 1:
			if( pUTF8Code[0] <= 127 ) return pUTF8Code[0];
			else break;
		case 2:
			if( pUTF8Code[0] >= 192 && pUTF8Code[0] < 224
				&& pUTF8Code[1] >= 128 && pUTF8Code[1] < 192
			) {
				n0 = pUTF8Code[0] - 192;
				n1 = pUTF8Code[1] - 128;
				return n0 * 64 + n1;
			} else break;
		case 3:
			if( pUTF8Code[0] >= 224 && pUTF8Code[0] < 240
				&& pUTF8Code[1] >= 128 && pUTF8Code[1] < 192
				&& pUTF8Code[2] >= 128 && pUTF8Code[2] < 192
			) {
				n0 = pUTF8Code[0] - 224;
				n1 = pUTF8Code[1] - 128;
				n2 = pUTF8Code[2] - 128;
				return n0 * 4096 + n1 * 64 + n2;
			} else break;
		default:
			return Bigint._error( "Bigint.uniocde.UTF8To16", "Input length must be 1, 2 or 3: " + pUTF8Code );
	}

	return Bigint._error( "Bigint.uniocde.UTF8To16", "Malformed input: " + pUTF8Code );

}



