
// number_theory.js

// __BIGINT_PRIMES = Bigint.getPrimeTable( 2, 10000 ); // Defined Below
//////////////////////////////////////////////////////////////////////////////////////



Bigint.prototype.isProbablePrime = function( base ) {
	var bigBase = Bigint.serio( base );

	if( this.table.length < 3 ) {
		var n = this.toNumber();
		var b = bigBase.toNumber();
		if( n < 2 ) return false;
		else if( n < 94906266 && b < 94906266 ) {
			return Bigint._isProbablePrime_typeN( n , b );
		}
		else if( n < 134217729 && b < 134217729 )
		{
			return Bigint._isProbablePrime_typeN2( n , b );
		}
	}

	var bigMinus1 = Bigint._sub_typeN( this , 1 );
	var bigTest = Bigint._powmod( bigBase, bigMinus1, this );


	return ( bigTest.table.length === 1 && bigTest.table[0] === 1 );
}

Bigint._isProbablePrime_typeN = function( n , base ) { // n must be <= 94,906,265 (2^(53/2))
	var power = n - 1;
	var strBinary = power.toString( 2 ); // Netscape 4 can do this too: max = [101101010000010011110011000]
	var lastIndex = strBinary.length - 1;
	var work = base;
	var result = ( strBinary.charAt( lastIndex ) === "1" )? base : 1;
	for( var i = lastIndex - 1; i > -1; i-- ) {
		work = work * work % n;
		if( strBinary.charAt(i) === "1" ) {
			result = result * work % n;
		}
	}
	return ( result === 1 );
}

Bigint._isProbablePrime_typeN2 = function( n , base ) { // modulus must be <= 134,217,728 (2^27)
	var power = n - 1;
	var strBinary = power.toString( 2 ); // Netscape 4 can do this ???
	var lastIndex = strBinary.length - 1;
	var work = base;
	var result = ( strBinary.charAt( lastIndex ) === "1" )? base : 1;

	for( var i = lastIndex - 1; i > -1; i-- ) {
		var w1, w2;
		if( work > __BIGINT_JS_INT_MAX_SQRT ) {
			if( work & 1 ) {
				w1 = ( work + 1 ) >>> 1;
				w2 = ( work - 1 ) >>> 1;
			} else {
				w1 = w2 = work >>> 1;
			}
			work = ( ( work * w1 % n ) + ( work * w2 % n ) ) % n;
		} else {
			work = work * work % n;
		}

		if( strBinary.charAt(i) === "1" ) {
			if( result * work > __BIGINT_JS_INT_MAX ) {
				if( work & 1 ) {
					w1 = ( work + 1 ) >>> 1;
					w2 = ( work - 1 ) >>> 1;
				} else {
					w1 = w2 = work >>> 1;
				}
				result = ( ( result * w1 % n ) + ( result * w2 % n ) ) % n;
			} else {
				result = result * work % n;
			}

		}

	}

	return ( result === 1 );
}



Bigint.prototype.isStrongProbablePrime = function( base ) {
	var bigBase = Bigint.serio( base );
	var bigOne = new Bigint( 1 );
//	if( this.isGreaterThan( 1 ) && bigBase.isGreaterThan( 1 ) )
	if( this._isGreaterThan( bigOne ) && bigBase._isGreaterThan( bigOne ) )
	{
		return this._isStrongProbablePrime( bigBase );
	}
	return false;
}

Bigint.prototype._isStrongProbablePrime = function( bigBase ) { // this should be >= 2
	var bigMinus1 = Bigint._sub_typeN( this , 1 );
	var bigA = bigMinus1.copy();

	for( var nMaxSteps = 0; bigA.isEven(); nMaxSteps++ ) {
		bigA.half();
	}

	var bigRemainder = Bigint._powmod( bigBase, bigA, this );

	if( bigRemainder.isEqualTo( 1 ) || nMaxSteps > 0 && bigRemainder.isEqualTo( bigMinus1 ) ) {
		return true;
	}
	for( var nSteps = 1; nSteps < nMaxSteps; nSteps++ ) {
		bigRemainder = Bigint._square( bigRemainder );
		bigRemainder = Bigint._mod_typeF( bigRemainder, this );
		if( bigRemainder.isEqualTo( bigMinus1 ) ) {
			return true;
		}
	}
	return false;
}

Bigint.prototype._is_3n = function() {
	var test = 0;
	var table = this.table;
	var len = table.length;
	for( var i = 0; i < len; i++ ) {
		test += table[i] % 3;
	}
	return ( test % 3 === 0 );
}

Bigint.prototype._is_5n = function() {
	return ( this.table[0] % 5 === 0 );
}

Bigint.prototype._eratosthenes = function() {
	var strBigNumber = this.toDecimalString();

	for( var i = 3; i < 669; i++ ) { // (__BIGINT_PRIMES[3] = 7, __BIGINT_PRIMES[668] = 4999
		if( Bigint._isDivisible_typeN( strBigNumber, __BIGINT_PRIMES[ i ] ) ) return true;
	}

	return false;
}

Bigint.prototype.eratosthenes = function()
{
	if( this.isEven() ) return 2;
	else if( this._is_3n() ) return 3;
	else if( this._is_5n() ) return 5;

	var strBigNumber = this.toDecimalString();
	for( var i = 3; i < 669; i++ ) {
		if( Bigint._isDivisible_typeN( strBigNumber, __BIGINT_PRIMES[ i ] ) ) return __BIGINT_PRIMES[ i ];
	}

	return 0;
}

Bigint.prototype.factorize = function( input ) {
	var nMax;
	if( input.isBigint ) nMax = input.toNumber();
	else if( input === undefined || input === "" ) nMax = 10000;
	else if( isNaN( input ) || input < 2 ) return Bigint._error( "factorize: bad input: " + input );
	else nMax = Number( input );

	if( this.isEqualTo( 0 ) ) return "0 = 0.";
	else if( this.isEqualTo( 1 ) ) return "1 = 1.";

	var sqrt = Math.floor( Math.sqrt( this.toNumber() ) );
	if( nMax > sqrt) nMax = sqrt;
	return this._factorize( nMax );
}

Bigint.gcd = function( arg1, arg2 )
{
	var big1 = Bigint.serio( arg1 );
	var big2 = Bigint.serio( arg2 );
	if( big1.isZero() ) return big2;
	else if( big2.isZero() ) return big1;

	return Bigint._gcd( big1, big2 );
}


Bigint._gcd = function( bigNumber1, bigNumber2 ) {
	var bigA, bigB;
	var test = Bigint._tablecmp( bigNumber1.table, bigNumber2.table );
	if( test >= 0 ) {
		bigA = bigNumber1.copy();
		bigB = bigNumber2.copy();
	} else {
		bigA = bigNumber2.copy();
		bigB = bigNumber1.copy();
	}
	for( var dummy=0; dummy<100; dummy++ ) {
		var bigR = Bigint._mod( bigA, bigB );
		if( bigR.isZero() ) return bigB;
		else {
			bigA = bigB.copy();
			bigB = bigR.copy();
		}
	}
	return Bigint._error( "_gcd", "too many loops for " + bigNumber1 + ", " + bigNumber2 );
}

Bigint._getK = function( n ) // LCM[ 1,2,...,n ] を返す
{
	var i, p, power;
	var sq = Math.sqrt( n );
	var bigResult = new Bigint( 1 );

	if( n < 9973 ) {
		for( i = 0; ; i++ )
		{
			p = __BIGINT_PRIMES[ i ];
			if( p > n ) break;
			power = Math.floor( Math.log( n ) / Math.log( p ) );
			bigResult = Bigint.mul( bigResult, Math.pow( p, power ) );
		}
	}
	else
	{
_timing();
		var myTable = Bigint.getPrimeTable( 2, n );
_timing("getPrimeTable");
		var len = myTable.length;
		for( i = 0; i < len; i++ )
		{
			p = myTable[ i ];
			power = Math.floor( Math.log( n ) / Math.log( p ) );
			bigResult = Bigint.mul( bigResult, Math.pow( p, power ) );
		}
		myTable = null;
_timing("bigResult");
	}

	return bigResult;
}
Bigint.getRepunit = function(n) {
	var strBinary = n.toString( 2 );
	var nLastIndex = strBinary.length - 1;
	var strResult = "";
	var strWork = "1";
	for( var i = nLastIndex; ; i-- ) {
		if( strBinary.charAt( i ) === "1" ) strResult += strWork;
		if( i === 0 ) break;
		strWork += strWork;
	}
	return new Bigint( strResult );
}

Bigint._revmod = function( bigNumber, bigModulus ) {

	var r = [ bigModulus.copy(), bigNumber.copy() ];
	var q = [ undefined, undefined ];
	var x = [ new Bigint(1), new Bigint(0) ];
	var y = [ new Bigint(0), new Bigint(1) ];

	for(var i=2; i<100; i++) {
		r[i] = Bigint.mod( r[i-2] , r[i-1] );
		if( r[i].isZero() ) {
			return Bigint.mod( y[i-1] , bigModulus );
		} else {
			var tableQR = Bigint.qr(   Bigint.sub(r[i-2],r[i]) , r[i-1]   );

			q[i] = tableQR[0];
			x[i] = Bigint.sub( x[i-2] ,   Bigint.mul(q[i],x[i-1])    );
			y[i] = Bigint.sub( y[i-2] ,   Bigint.mul(q[i],y[i-1])    );
		}
	}

	return Bigint._error( "_revmod" , "too many loops for " + bigNumber + ", " + bigModulus );
}


Bigint._getFactors = function( n ) {
	var tableResult = new Array();
	var idx = -1;
	var a = n;
	while( true ) {
		var p = Bigint._getFactor10000( a );
		tableResult[ ++idx ] = p;
		if( p === a ) break;
		a /= p;
	}
	return tableResult;
}

Bigint._getFactor10000 = function( n ) {
	var sqrt = Math.sqrt( n );
//	if( sqrt > 10000 ) {
//		return Bigint._error( "_getFactor10000" , "Input greater than 10000^2: " + n );
//	}

	for( var i = 0; i < 1230; i++ ) { // 1229 primes < 10000; The last one in the list is 1230th = 10007
		var p = __BIGINT_PRIMES[ i ];
		if( p > sqrt ) return n;
		if( n % p === 0 ) return p;
	}
	return "?";
}

Bigint._isCarmichaelNumber = function( n ) {
	if( ( n & 1 ) === 0 || n < 0 ) return false;
	var tableFactors = new Array();
	var idx = -1;
	var a = n;
	while( true ) {
		var p = Bigint._getFactor10000( a );

		if( idx > -1 && p === tableFactors[ idx ] ) {
			return false; // not square-frees
		}

		tableFactors[ ++idx ] = p;
		if( p === a ) break;
		a /= p;
	}

	var len = tableFactors.length;
	if( len < 3 ) return false; // prime or 2-factor
	var n_1 = n - 1;
	for( var i = 0; i < len; i++ ) {
		if( n_1 % ( tableFactors[ i ] - 1 ) ) return false;
	}
	return true;//tableFactors;
}


Bigint.countPrimes = function( limit ) {
	defaultStatus = "0%";
	var nLimit = Bigint.parseNumber( limit );

	if( nLimit === __BIGINT_INVALID_NUMBER ) {
		return Bigint._error( "countPrimes" , "Bad argument: " + limit );
	}

	if( nLimit < 2 ) return 0;

	var UNIT = 500;

	var sqrt = Math.sqrt( nLimit );
	var tableSmallPrimes = Bigint._getPrimeTableLessThan( sqrt );

_debug("tableSmallPrimes.length",tableSmallPrimes.length);

	var count = 1;

	var cnt = 0;

	for( var start = 3; start < nLimit; start += UNIT, cnt++ ) {
		if( cnt === 100 ) {
			var percent = start / nLimit * 100;
			/*-- debug for checking time cost
			if( percent >= 1 ) return count;
			---*/
			defaultStatus = percent + "%";
			cnt = 0;
		}

		var end = start + UNIT - 1;
		if( end > nLimit ) end = nLimit;

		count += Bigint._getOddPrimeCountRange( start, end, tableSmallPrimes );

	}
	defaultStatus = "";
	return count;
}

/* -------------------------------------------------------- */
/* PI( 10000000000) = 455052512, PI(     316241) =    27294 */
/* PI(  1000000000) =  50847534, PRIME(   27293) =   316223 */
/* PI(   100000000) =   5761455, 316241 * 316241 > 10 ** 11 */
/* PI(    10000000) =    664579, 316223 * 316223 < 10 ** 11 */
/* PI(     1000000) =     78498, PRIME( 1000000) = 15485863 */
/* PI(      100000) =      9592, PRIME(  100000) =  1299709 */
/* PI(       10000) =      1229, PRIME(   10000) =   104729 */
/* PI(        1000) =       168, PRIME(    1000) =     7919 */
/* PI(         100) =        25, PRIME(     100) =      541 */
/* PI(          10) =         4, PRIME(      10) =       29 */
/* PI(           1) =         0, PRIME(       1) =        2 */


Bigint.getPrimeTable = function( min, max, tableSmallPrimes ) {
	if( max === void 0 ) { max = min, min = 0; }

	var UNIT = 500;

	var nMin = Bigint.parseNumber( min );
	var nMax = Bigint.parseNumber( max );
	if( __BIGINT_INVALID_NUMBER === nMin || __BIGINT_INVALID_NUMBER === nMax )
	{
		return Bigint._error( "getPrimeTable" , "Bad argument(s): " + nMin + ", " + nMax );
	}

	if( nMax < 2 || nMin > nMax ) return new Array();

	if( tableSmallPrimes === undefined ) {
		tableSmallPrimes = Bigint._getPrimeTableLessThan( Math.sqrt( nMax ) );
	}
	var tableResult = new Array();
	var idx = -1;
	if( nMin <= 2 ) {
		tableResult[ 0 ] = 2;
		idx = 0;
	}

	var cnt = 0;
//	var UNIT = 500;
	for( var start = nMin; start <= nMax; start += UNIT, cnt++ ) {
		if( cnt === 100 ) {
			defaultStatus = ( start / nMax ) + " / 1.000";
			cnt = 0;
		}
		var end = start + UNIT - 1;
		if( end > nMax ) end = nMax;
		var tmpTable = Bigint._getOddPrimeTableRange( start, end, tableSmallPrimes );
		var len = tmpTable.length;
		for( var i = 0; i < len; i++ ) {
			tableResult[ ++idx ] = tmpTable[ i ];
		}
	}
	defaultStatus = "";
	return tableResult;
}

Bigint._getOddPrimeTableRange = function( nFrom, nTo, tableSmallPrimes ) {
	var MARKED_AS_COMPOSITE = -1;
	if( tableSmallPrimes === void 0 ) tableSmallPrimes = Bigint._getPrimeTableLessThan( Math.sqrt( nTo ) );
	if( nFrom < 3 ) nFrom = 3;
	else if( ( nFrom & 1 ) === 0 ) nFrom++;
	var oWork = new Object();
	var len = tableSmallPrimes.length;
	for( var i = 1; i < len; i++ ) { // i = 0 ( for prime 2 ) is not used
		var p = tableSmallPrimes[ i ];
		var r = nFrom % p;
		var q = ( nFrom - r ) / p;
		var start;
		
		if( r === 0 ) {
			if( q < 2 ) start = p * 2;
			else start = p * q;
		} else {
			if( q < 2 ) start = p * 2;
			else start = p * ( q + 1 );
		}
		
		for( var j = start; j <= nTo; j += p ) {
			oWork[ j ] = MARKED_AS_COMPOSITE;
		}
	}

	var tablePrimes = new Array();
	var idx = -1;
	for( i = nFrom; i <= nTo; i+=2 ) {
		if( oWork[ i ] !== MARKED_AS_COMPOSITE ) {
			tablePrimes[ ++idx ] = i;
		}
	}
	oWork = null;
	return tablePrimes;
}


Bigint._getOddPrimeCountRange = function( nFrom, nTo, tableSmallPrimes ) {
//	var NOT_MARKED = void 0;
	var MARKED_AS_COMPOSITE = -1;
	if( tableSmallPrimes === void 0 ) tableSmallPrimes = Bigint._getPrimeTableLessThan( Math.sqrt( nTo ) );
	if( nFrom < 3 ) nFrom = 3;
	else if( ( nFrom & 1 ) === 0 ) nFrom++;
	var oWork = new Object();

	var len = tableSmallPrimes.length;
	for( var i = 1; i < len; i++ ) { // i = 0 ( for prime 2 ) is not used
		var p = tableSmallPrimes[ i ];
		var r = nFrom % p;
		var q = ( nFrom - r ) / p;
		var start;
		if( r === 0 ) {
			if( q < 2 ) start = p * 2;
			else start = p * q;
		} else {
			if( q < 2 ) start = p * 2;
			else start = p * ( q + 1 );
		}
		for( var j = start; j <= nTo; j += p ) {
			oWork[ j ] = MARKED_AS_COMPOSITE;
		}
	}
	var tablePrimes = new Array();
	var count = 0;
	for( i = nFrom; i <= nTo; i+=2 ) {
		if( oWork[ i ] !== MARKED_AS_COMPOSITE ) {
			count++;
		}
	}
	oWork = null;
	return count;
}

Bigint._getOddCompositeTableRange = function( nFrom, nTo, tableSmallPrimes ) {
	var MARKED_AS_COMPOSITE = -1;
	if( tableSmallPrimes === void 0 ) tableSmallPrimes = Bigint._getPrimeTableLessThan( Math.sqrt( nTo ) );
	if( nFrom < 3 ) nFrom = 3;
	else if( ( nFrom & 1 ) === 0 ) nFrom++;
	var oWork = new Object();

	var len = tableSmallPrimes.length;
	for( var i = 1; i < len; i++ ) { // i = 0 ( for prime 2 ) is not used
		var p = tableSmallPrimes[ i ];
		var r = nFrom % p;
		var q = ( nFrom - r ) / p;
		var start;
		
		if( r === 0 ) {
			if( q < 2 ) start = p * 2;
			else start = p * q;
		} else {
			if( q < 2 ) start = p * 2;
			else start = p * ( q + 1 );
		}
		
		for( var j = start; j <= nTo; j += p ) {
			oWork[ j ] = MARKED_AS_COMPOSITE;
		}
	}

	var tableComposite = new Array();
	var idx = -1;
	for( i = nFrom; i <= nTo; i+=2 ) {
		if( oWork[ i ] === MARKED_AS_COMPOSITE ) {
			tableComposite[ ++idx ] = i;
		}
	}
	oWork = null;
	return tableComposite;
}



Bigint._getPrimeTableLessThan = function( nMax ) {
	var NOT_MARKED = void 0;
	var MARKED_AS_PRIME = +1;
	var MARKED_AS_COMPOSITE = -1;

	if( nMax < 2 ) return new Array();
	else if( nMax > 1000000 ) {
		return Bigint._error( "getPrimeTable" , "nMax too big: " + nMax );
	}
	var oWork = new Object();
//	for( var i = 3; i < nMax; i += 2 ) {
//		oWork[ i ] = NOT_MARKED;
//	}

	var sqrt = Math.sqrt( nMax );
	var p = 3;

	Eratosthenes: for( var n = 3; ; n = p + 2 ) {
		// Finding the next prime p
		for( var i = n; ; i += 2 ) {
			if( i > sqrt ) break Eratosthenes;
			if( oWork[ i ] === NOT_MARKED ) {
				oWork[ i ] = MARKED_AS_PRIME;
				p = i;
				break;
			}
		}
		// Marked 2p, 3p, ... as composite
		for( i = 2 * p; i <= nMax; i += p ) {
			oWork[ i ] = MARKED_AS_COMPOSITE;
		}
	}

	// Making the table
	var tableResult = [ 2 ];
	var idx = 0;
	for( i = 3; i <= nMax; i+= 2 ) {
		if( oWork[ i ] !== MARKED_AS_COMPOSITE ) {
			tableResult[ ++idx ] = i;
		}
	}
	oWork = null;
	return tableResult;
}



Bigint.getPseudoprimeTable = function( min, max, base ) {
	var nMin = Bigint.parseNumber( min );
	var nMax = Bigint.parseNumber( max );
	var nBase = Bigint.parseNumber( base );

	if( nMin === __BIGINT_INVALID_NUMBER || nMax === __BIGINT_INVALID_NUMBER || nBase === __BIGINT_INVALID_NUMBER 
	||
	nBase !== Math.floor( nBase )
	||
	nBase < 2
	) {
		return Bigint._error( "getPseudoprimeTable" , "Bad argument(s): " + min + ", " + max + ", " + base );
	}

	if( nMin < 0 ) nMin = 0;
	if( nMin > nMax ) return new Array();
	if( nMax > 134217728 ) {
		return Bigint._error( "getPseudoprimeTable" , "nMax too big: " + nMax );
	}

	var UNIT = 500;

	var tableSmallPrimes = Bigint.getPrimeTable( 0, Math.sqrt( nMax ) );

	var tableResult = new Array();

	var cnt = 0;
	var progressZero = ( nMin > 1 )? nMin : 1 ;
	var progressTotal = ( nMin > 1 )? nMax - nMin : nMax ;
	var progressParam = (__BIGINT_MOZ)? 5 : 20 ;
	if( nMax <= 94906265 ) {
		for( var start = nMin; start <= nMax; start += UNIT, cnt++ ) {

			if( cnt === progressParam ) {
				defaultStatus = ( ( start - progressZero ) / progressTotal ) + " / 1.000";
				cnt = 0;
			}

			var end = start + UNIT - 1;
			if( end > nMax ) end = nMax;

			var tableComposite = Bigint._getOddCompositeTableRange( start, end, tableSmallPrimes );


			var tableWork = new Array();
			var idx = -1;
			var len = tableComposite.length;
			for( var i = 0; i < len; i++ ) {
				var n = tableComposite[ i ];
				if( Bigint._isProbablePrime_typeN( n, nBase ) ) { tableWork[ ++idx ] = n; }
			}

			tableResult = tableResult.concat( tableWork );
		}
	} else {
		for( start = nMin; start <= 94906000 && start <= nMax; start += UNIT, cnt++ ) {

			if( cnt === progressParam ) {
				defaultStatus = ( ( start - progressZero ) / progressTotal ) + " / 1.000";
				cnt = 0;
			}

			end = start + UNIT - 1;
			if( end > nMax ) end = nMax;

			tableComposite = Bigint._getOddCompositeTableRange( start, end, tableSmallPrimes );

			tableWork = new Array();
			idx = -1;
			len = tableComposite.length;
			for( i = 0; i < len; i++ ) {
				n = tableComposite[ i ];
				if( Bigint._isProbablePrime_typeN( n, nBase ) ) { tableWork[ ++idx ] = n; }
			}

			tableResult = tableResult.concat( tableWork );
		}

		var nLoop2Init = ( nMin <= 94906001 )? 94906001 : nMin;

		for( start = nLoop2Init; start <= nMax; start += UNIT, cnt++ ) {

			if( cnt === progressParam ) {
				defaultStatus = ( ( start - progressZero ) / progressTotal ) + " / 1.000";
				cnt = 0;
			}

			end = start + UNIT - 1;
			if( end > nMax ) end = nMax;

			tableComposite = Bigint._getOddCompositeTableRange( start, end, tableSmallPrimes );

			tableWork = new Array();
			idx = -1;
			len = tableComposite.length;
			for( i = 0; i < len; i++ ) {
				n = tableComposite[ i ];



				if( Bigint._isProbablePrime_typeN2( n, nBase ) ) { 
					tableWork[ ++idx ] = n;
				}


			}
			tableResult = tableResult.concat( tableWork );
		}
	}

	return tableResult;

}






//////////////////////////////////////////////////////////////////////////////////////
var __BIGINT_PRIMES = [ 2, 3, 5 , 7 , 11 , 13 , 17 , 19 , 23 , 29 , 31 , 37 , 41 , 43 , 47 , 53 , 59 ,
	 61 , 67 , 71 , 73 , 79 , 83 , 89 , 97 , 101 , 103 , 107 , 109 , 113 , 127 , 131 , 137 ,
	 139 , 149 , 151 , 157 , 163 , 167 , 173 , 179 , 181 , 191 , 193 , 197 , 199 , 211 , 223 ,
	 227 , 229 , 233 , 239 , 241 , 251 , 257 , 263 , 269 , 271 , 277 , 281 , 283 , 293 , 307 ,
	 311 , 313 , 317 , 331 , 337 , 347 , 349 , 353 , 359 , 367 , 373 , 379 , 383 , 389 , 397 ,
	 401 , 409 , 419 , 421 , 431 , 433 , 439 , 443 , 449 , 457 , 461 , 463 , 467 , 479 , 487 ,
	 491 , 499 , 503 , 509 , 521 , 523 , 541 , 547 , 557 , 563 , 569 , 571 , 577 , 587 , 593 ,
	 599 , 601 , 607 , 613 , 617 , 619 , 631 , 641 , 643 , 647 , 653 , 659 , 661 , 673 , 677 ,
	 683 , 691 , 701 , 709 , 719 , 727 , 733 , 739 , 743 , 751 , 757 , 761 , 769 , 773 , 787 ,
	 797 , 809 , 811 , 821 , 823 , 827 , 829 , 839 , 853 , 857 , 859 , 863 , 877 , 881 , 883 ,
	 887 , 907 , 911 , 919 , 929 , 937 , 941 , 947 , 953 , 967 , 971 , 977 , 983 , 991 , 997,
	1009, 1013, 1019, 1021, 1031, 1033, 1039, 1049, 1051, 1061, 1063, 
1069, 1087, 1091, 1093, 1097, 1103, 1109, 1117, 1123, 1129, 1151, 1153, 1163, 1171, 1181, 1187, 1193, 1201, 
1213, 1217, 1223, 1229, 1231, 1237, 1249, 1259, 1277, 1279, 1283, 1289, 1291, 1297, 1301, 1303, 1307, 1319, 1321, 1327, 
1361, 1367, 1373, 1381, 1399, 1409, 1423, 1427, 1429, 1433, 1439, 1447, 1451, 1453, 1459, 1471, 1481, 1483, 1487, 1489, 
1493, 1499, 1511, 1523, 1531, 1543, 1549, 1553, 1559, 1567, 1571, 1579, 1583, 1597, 1601, 1607, 1609, 1613, 1619, 1621, 1627, 
1637, 1657, 1663, 1667, 1669, 1693, 1697, 1699, 1709, 1721, 1723, 1733, 1741, 1747, 1753, 1759, 
1777, 1783, 1787, 1789, 1801, 1811, 1823, 1831, 1847, 1861, 1867, 1871, 1873, 1877, 1879, 1889, 1901, 1907, 1913, 
1931, 1933, 1949, 1951, 1973, 1979, 1987, 1993, 1997, 1999, 2003, 2011, 2017, 2027, 2029, 2039, 2053, 
2063, 2069, 2081, 2083, 2087, 2089, 2099, 2111, 2113, 2129, 2131, 2137, 2141, 2143, 2153, 2161, 2179, 
2203, 2207, 2213, 2221, 2237, 2239, 2243, 2251, 2267, 2269, 2273, 2281, 2287, 2293, 2297, 2309, 2311, 2333, 2339, 2341, 
2347, 2351, 2357, 2371, 2377, 2381, 2383, 2389, 2393, 2399, 2411, 2417, 2423, 2437, 2441, 2447, 2459, 2467, 2473, 2477, 
2503, 2521, 2531, 2539, 2543, 2549, 2551, 2557, 2579, 2591, 2593, 2609, 2617, 2621, 
2633, 2647, 2657, 2659, 2663, 2671, 2677, 2683, 2687, 2689, 2693, 2699, 2707, 2711, 2713, 2719, 2729, 2731, 2741, 2749, 2753, 2767, 
2777, 2789, 2791, 2797, 2801, 2803, 2819, 2833, 2837, 2843, 2851, 2857, 2861, 2879, 2887, 2897, 2903, 2909, 
2917, 2927, 2939, 2953, 2957, 2963, 2969, 2971, 2999, 3001, 3011, 3019, 3023, 3037, 3041, 3049, 
3061, 3067, 3079, 3083, 3089, 3109, 3119, 3121, 3137, 3163, 3167, 3169, 3181, 3187, 3191, 
3203, 3209, 3217, 3221, 3229, 3251, 3253, 3257, 3259, 3271, 3299, 3301, 3307, 3313, 3319, 3323, 3329, 3331, 
3343, 3347, 3359, 3361, 3371, 3373, 3389, 3391, 3407, 3413, 3433, 3449, 3457, 3461, 3463, 3467, 3469, 
3491, 3499, 3511, 3517, 3527, 3529, 3533, 3539, 3541, 3547, 3557, 3559, 3571, 3581, 3583, 3593, 3607, 3613, 3617, 
3623, 3631, 3637, 3643, 3659, 3671, 3673, 3677, 3691, 3697, 3701, 3709, 3719, 3727, 3733, 3739, 3761, 
3767, 3769, 3779, 3793, 3797, 3803, 3821, 3823, 3833, 3847, 3851, 3853, 3863, 3877, 3881, 3889, 
3907, 3911, 3917, 3919, 3923, 3929, 3931, 3943, 3947, 3967, 3989, 4001, 4003, 4007, 4013, 4019, 4021, 4027, 
4049, 4051, 4057, 4073, 4079, 4091, 4093, 4099, 4111, 4127, 4129, 4133, 4139, 4153, 4157, 4159, 4177, 
4201, 4211, 4217, 4219, 4229, 4231, 4241, 4243, 4253, 4259, 4261, 4271, 4273, 4283, 4289, 4297, 4327, 
4337, 4339, 4349, 4357, 4363, 4373, 4391, 4397, 4409, 4421, 4423, 4441, 4447, 4451, 4457, 4463, 
4481, 4483, 4493, 4507, 4513, 4517, 4519, 4523, 4547, 4549, 4561, 4567, 4583, 4591, 4597, 4603, 
4621, 4637, 4639, 4643, 4649, 4651, 4657, 4663, 4673, 4679, 4691, 4703, 4721, 4723, 4729, 4733, 4751, 
4759, 4783, 4787, 4789, 4793, 4799, 4801, 4813, 4817, 4831, 4861, 4871, 4877, 4889, 
4903, 4909, 4919, 4931, 4933, 4937, 4943, 4951, 4957, 4967, 4969, 4973, 4987, 4993, 4999, 5003, 5009, 5011, 5021, 5023, 5039, 
5051, 5059, 5077, 5081, 5087, 5099, 5101, 5107, 5113, 5119, 5147, 5153, 5167, 5171, 5179, 
5189, 5197, 5209, 5227, 5231, 5233, 5237, 5261, 5273, 5279, 5281, 5297, 5303, 5309, 5323, 
5333, 5347, 5351, 5381, 5387, 5393, 5399, 5407, 5413, 5417, 5419, 5431, 5437, 5441, 5443, 5449, 
5471, 5477, 5479, 5483, 5501, 5503, 5507, 5519, 5521, 5527, 5531, 5557, 5563, 5569, 5573, 5581, 5591, 
5623, 5639, 5641, 5647, 5651, 5653, 5657, 5659, 5669, 5683, 5689, 5693, 5701, 5711, 5717, 5737, 5741, 5743, 5749, 
5779, 5783, 5791, 5801, 5807, 5813, 5821, 5827, 5839, 5843, 5849, 5851, 5857, 5861, 5867, 5869, 5879, 5881, 
5897, 5903, 5923, 5927, 5939, 5953, 5981, 5987, 6007, 6011, 6029, 
6037, 6043, 6047, 6053, 6067, 6073, 6079, 6089, 6091, 6101, 6113, 6121, 6131, 6133, 6143, 6151, 6163, 6173, 
6197, 6199, 6203, 6211, 6217, 6221, 6229, 6247, 6257, 6263, 6269, 6271, 6277, 6287, 6299, 6301, 6311, 6317, 
6323, 6329, 6337, 6343, 6353, 6359, 6361, 6367, 6373, 6379, 6389, 6397, 6421, 6427, 6449, 6451, 
6469, 6473, 6481, 6491, 6521, 6529, 6547, 6551, 6553, 6563, 6569, 6571, 6577, 6581, 6599, 
6607, 6619, 6637, 6653, 6659, 6661, 6673, 6679, 6689, 6691, 6701, 6703, 6709, 6719, 6733, 6737, 
6761, 6763, 6779, 6781, 6791, 6793, 6803, 6823, 6827, 6829, 6833, 6841, 6857, 6863, 6869, 6871, 6883, 
6899, 6907, 6911, 6917, 6947, 6949, 6959, 6961, 6967, 6971, 6977, 6983, 6991, 6997, 7001, 7013, 7019, 7027, 
7039, 7043, 7057, 7069, 7079, 7103, 7109, 7121, 7127, 7129, 7151, 7159, 
7177, 7187, 7193, 7207, 7211, 7213, 7219, 7229, 7237, 7243, 7247, 7253, 7283, 7297, 7307, 7309, 
7321, 7331, 7333, 7349, 7351, 7369, 7393, 7411, 7417, 7433, 7451, 
7457, 7459, 7477, 7481, 7487, 7489, 7499, 7507, 7517, 7523, 7529, 7537, 7541, 7547, 7549, 7559, 7561, 7573, 7577, 7583, 7589, 7591, 
7603, 7607, 7621, 7639, 7643, 7649, 7669, 7673, 7681, 7687, 7691, 7699, 7703, 7717, 7723, 7727, 
7741, 7753, 7757, 7759, 7789, 7793, 7817, 7823, 7829, 7841, 7853, 7867, 7873, 7877, 7879, 
7883, 7901, 7907, 7919, 7927, 7933, 7937, 7949, 7951, 7963, 7993, 8009, 8011, 8017, 
8039, 8053, 8059, 8069, 8081, 8087, 8089, 8093, 8101, 8111, 8117, 8123, 8147, 8161, 
8167, 8171, 8179, 8191, 8209, 8219, 8221, 8231, 8233, 8237, 8243, 8263, 8269, 8273, 8287, 8291, 8293, 8297, 
8311, 8317, 8329, 8353, 8363, 8369, 8377, 8387, 8389, 8419, 8423, 8429, 8431, 8443, 8447, 
8461, 8467, 8501, 8513, 8521, 8527, 8537, 8539, 8543, 8563, 8573, 8581, 
8597, 8599, 8609, 8623, 8627, 8629, 8641, 8647, 8663, 8669, 8677, 8681, 8689, 8693, 8699, 8707, 8713, 8719, 8731, 
8737, 8741, 8747, 8753, 8761, 8779, 8783, 8803, 8807, 8819, 8821, 8831, 8837, 8839, 8849, 8861, 8863, 8867, 
8887, 8893, 8923, 8929, 8933, 8941, 8951, 8963, 8969, 8971, 8999, 9001, 9007, 9011, 9013, 
9029, 9041, 9043, 9049, 9059, 9067, 9091, 9103, 9109, 9127, 9133, 9137, 9151, 9157, 
9161, 9173, 9181, 9187, 9199, 9203, 9209, 9221, 9227, 9239, 9241, 9257, 9277, 9281, 9283, 9293, 
9311, 9319, 9323, 9337, 9341, 9343, 9349, 9371, 9377, 9391, 9397, 9403, 9413, 9419, 9421, 9431, 9433, 9437, 9439, 
9461, 9463, 9467, 9473, 9479, 9491, 9497, 9511, 9521, 9533, 9539, 9547, 9551, 
9587, 9601, 9613, 9619, 9623, 9629, 9631, 9643, 9649, 9661, 9677, 9679, 9689, 9697, 9719, 9721, 
9733, 9739, 9743, 9749, 9767, 9769, 9781, 9787, 9791, 9803, 9811, 9817, 9829, 9833, 9839, 9851, 9857, 9859, 
9871, 9883, 9887, 9901, 9907, 9923, 9929, 9931, 9941, 9949, 9967, 9973  ];






