// pigpgp.js
// v 0.2.4
//
//
//



var __BIGINT_PIG_HEADER_PUB = "PUB_PIGv2";
var __BIGINT_PIG_HEADER_SEC = "SEC_PIGv2";


var __BIGINT_PIG_TABLE = [
	"0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "a", "b", "c", "d", "e", "f",
	"g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v",
	"w", "x", "y", "z", "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L",
	"M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z", "!", "#",
	"$", "%", "(", ")", "*", "+", ",", "-", ".", "/", ":", ";", "=", "?", "^", "_",
	"`", "{", "|", "}", "~", "[A", "[B", "[C", "[D", "[E",
	 "[F", "[G", "[H", "[I", "[J", "[K", "[L", "[M", "[N", "[O"
];
var __BIGINT_PIG_RADIX = __BIGINT_PIG_TABLE.length;
var __BIGINT_bigPIG_RADIX = new Bigint(__BIGINT_PIG_RADIX);

var __BIGINT_UNPIG = new Object;
for( var __BIGINT_i = 0; __BIGINT_i < __BIGINT_PIG_RADIX; __BIGINT_i++ ) {
	__BIGINT_UNPIG[ __BIGINT_PIG_TABLE[ __BIGINT_i ] ] = __BIGINT_i;
}


Bigint.prototype.toPigString = function() {
	if( !this.isNonNegative ) return Bigint._error( "toPigString" , "input is negative: " + this );

	var strNumber = this.toDecimalString();
	return Bigint._toPigString( strNumber );
}

Bigint._toPigString = function( strNumber ) {
	var strResult;
	var pos;
	var len = strNumber.length;
	if( len & 0x1 ) {
		strResult = __BIGINT_PIG_TABLE[ Number( strNumber.charAt(0) ) ];
		pos = 1;
	} else {
		strResult = __BIGINT_PIG_TABLE[ Number( strNumber.substr(0 , 2) ) ];
		pos = 2;
	}

	for( var i = pos; i < len; i+=2 ) {
		strResult += __BIGINT_PIG_TABLE[ Number( strNumber.substr( i , 2 ) ) ];
	}

	return strResult;

}

Bigint._fromPigString = function( strPig ) {
	var str = Bigint._pigStringToString( strPig );
	return new Bigint( str );
}

Bigint._pigStringToString = function( strPig ) {
	var strResult = "";
	var c;
	var len = strPig.length;
	for( var i = 0; i < len; i++ ) {
		c = strPig.charAt( i );
		if( c === "[" ) c += strPig.charAt( ++i );
		var nUnpig = __BIGINT_UNPIG[ c ];
		if( nUnpig !== undefined ) {
			strResult += Bigint._padding2( nUnpig );
		} else {
			if( c.length > 1 ) c = c.charAt( 1 );
			Bigint._error( "_pigStringToString" , "illegal character: " + c + ": code=" + c.charCodeAt(0) + ": input=" + strPig );
			return undefined;
		}
	}
	return strResult;
}

//
// CONSTRUCTOR :: PigPGP
//

PigPGP = function( strHeader, bigPower, bigModulus ) {
	if( strHeader === "PUB_PIGv2" || strHeader === "SEC_PIGv2" ) {
		this.header = strHeader;
	} else {
		return Bigint._error( "PigPGP (constructor)" , "Header corrupted: " + strHeader );
	}

	if( bigPower.isBigint ) {
		this.power = bigPower;
	} else {
		return Bigint._error( "PigPGP (constructor)" , "bigPower " + bigPower + " is not Bigint" );
	}
	
	if( bigModulus.isBigint ) {
		this.modulus = bigModulus;
	} else {
		return Bigint._error( "PigPGP (constructor)" , "bigModulus " + bigModulus + " is not Bigint" );
	}
	
	this.asciiString =
		PigPGP._formatKeyString( this.header + "@" + this.power.toPigString() + "@" + this.modulus.toPigString(), "]" );
	this.isPigKey = true;
	return true;
}

PigPGP._formatKeyString = function( strInput, strTerminator ) {
	if( !strInput || typeof strInput !== "string" ) {
		return Bigint._error( "PigPGP._formatKeyString" , "Input void or bad: " + strInput );
	}
	if( !strTerminator ) strTerminator = "";
	var strResult= "";
	var len = strInput.length;
	for( var i = 0, cnt = 0; i < len; i++, cnt++ ) {
		strResult += strInput.charAt( i );
		if( cnt === 60 && i !== len - 1 ) {
			strResult += "\n";
			cnt = 0;
		}
	}
	return strResult + strTerminator;
}


PigPGP.prototype.toString = function() {
	var strAbout = "(PigPGP v0.2)";
	if( this.header === "PUB_PIGv2" ) {
		strAbout = "RSA Public Key " + strAbout;
	} else if( this.header === "SEC_PIGv2" ) {
		strAbout = "RSA Secret Key " + strAbout;
	} else {
		return "Unknown Key: " + this.header;
	}

	return strAbout + "\n"
		+ "Key length: " + this.modulus.getBitWidthS(2) + "-bit\n"
		+ "power: " + this.power + "\n"
		+ "modulus: " + this.modulus + "\n";
}

PigPGP.prototype.convert = function( input ) {
	var bigInput = Bigint.serio( input );
	if( bigInput.comparedTo( this.modulus ) >= 0 ) {
		return Bigint._error("PigPGP.prototype.convert", "Input is greater than Modulus: " + bigInput + ", " + this.modulus );
	} else {
		return Bigint._powmod( bigInput, this.power, this.modulus );
	}
}

PigPGP.prototype.encrypt = function( input ) {
	var bigInput = Bigint.serio( input );

	Bigint._debug( String.fromCharCode( 0x2605 ) + " Session Key" ,  bigInput);

	if( bigInput.comparedTo( this.modulus ) >= 0 ) {
		return Bigint._error("PigPGP.prototype.encrypt", "Input is greater than Modulus: " + bigInput + ", " + this.modulus );
	} else {

		Bigint._debug("Executing " + bigInput + " ^ " + this.power + " mod " + this.modulus);

		var bigResult = Bigint._powmod( bigInput, this.power, this.modulus );

		Bigint._debug("Session Key ( RSA-Encrypted )" , bigResult);
		return bigResult.toPigString();
	}
}


PigPGP.prototype.decrypt = function( pigString ) {
	var bigInput = Bigint._fromPigString( pigString );

/*
	NOTE: You can generate keys equal to 1024 bits or greater: However, in that case,
	this function should call PigPGP._powmod_tunedS instead of PigPGP._powmod_tunedF,
	as we need a slightly different (less efficient) algorithm for Modulus operation if the modulus is equal to 1024 bits or greater;
	because the biggest Float number JavaScript can handle by itself is smaller than 2**1024

	Though the standard library function, Bigint.powmod, would switch these algorithms automatically,
	we are here calling lower-level funtions directly for higher speed.
*/

	Bigint._debug("Session Key: ( RSA-Encrypted )" , bigInput );
	Bigint._debug("Executing " + bigInput + " ^ " + this.power + " mod " + this.modulus);

	var bigResult = ( this.modulus.table.length <= __BIGINT_JS_COMPATIBLE_MAX_CELLS )?
		PigPGP._powmod_tunedF( bigInput, this.power, this.modulus ) :
		PigPGP._powmod_tunedS( bigInput, this.power, this.modulus ) ;

	Bigint._debug( String.fromCharCode( 0x2605 ) + " Session Key" , bigResult );

	return bigResult;

}

PigPGP._parsePigKey = function( strInput ) {
	if( !strInput || typeof strInput !== "string" ) {
		return Bigint._error( "PigPGP._parsePigKey", "Void or bad input: " + strInput );
	}

	strInput = strInput.replace( /\s/g, "" );
	var lastIndex = strInput.length - 1;

	var pos1 = strInput.indexOf( "@" );
	var pos2 = strInput.lastIndexOf( "@" );
	var strHeader = strInput.substring( 0, pos1 );
	var strPower = strInput.substring( pos1 + 1, pos2 );
	var strModulus = strInput.substring( pos2 + 1, lastIndex );

	if( strHeader !== "PUB_PIGv2" && strHeader !== "SEC_PIGv2" || strInput.charAt( lastIndex ) !== "]" ) {
		return Bigint._error( "PigPGP._parsePigKey", "Key corrupted: " + strHeader + " : " + strPower + " : " + strModulus );
	}

	return new PigPGP(
		strHeader,
		Bigint._fromPigString( strPower ),
		Bigint._fromPigString( strModulus )
	);
}



PigPGP._stringToPackedData = function( strInput ) {

	if( strInput.search(/\S/) === -1 ) {
		return Bigint._error( "PigPGP._stringToPackedData" , "input is practically null" );
	}

	if( strInput.indexOf( "\r\n" ) < 0 ) {
		strInput = strInput.replace( /[\r\n]/g , "\r\n" );
	}


	var nMask = 0;

	var nChecksum = 0;

	var pMaskedData = new Array();
	////var pos = 0;
	var len = strInput.length
	for( var i = 0; i < len; i++ ) {
		var nCharCode = strInput.charCodeAt( i );
		if( nCharCode >= 65536 ) {
			return Bigint._error( "PigPGP._stringToMaskedData",
				"charCode GTE 65536: " + nCharCode + " for " + strInput.charAt(i) );
		}

		nChecksum += nCharCode;
		nChecksum %= 100;

		nMask += 9973;
		nMask %= 65536;
		var nMaskedCharCode = ( nCharCode + nMask ) % 65536;
		var nLow = nMaskedCharCode % 256;
//		var nHigh = ( nMaskedCharCode - nLow ) / 256;
		var nHigh = ( nMaskedCharCode - nLow ) >> 8;
		pMaskedData[i] = nLow * 256 + nHigh;
	}

	len = pMaskedData.length;
	if( len & 0x1 ) pMaskedData[ pMaskedData.length ] = 0;

	var pPacked = new Array();
	var pos = 0;
	for( i = 0; i < len; i+=2 ) {
		pPacked[pos++] = pMaskedData[i] * 65536 + pMaskedData[i+1];
	}
	
	var oPacked = new Object();
	oPacked.data = pPacked;
	if( strInput.length > 64 ) oPacked.checksum = nChecksum;
	else oPacked.checksum = undefined;

	return oPacked;
}

PigPGP._packedDataToString = function( oPacked ) {
	var pInput = oPacked.data;
	var pMaskedData = new Array();
	var pos = 0;
	var len = pInput.length;
	for( var i=0; i < len ; i++ ) {
		var n = pInput[i];
		var r = n % 65536;
//		pMaskedData[pos] = ( n - r ) / 65536;
		pMaskedData[pos] = ( n - r ) >>> 16;
		pMaskedData[pos+1] = r;
		pos += 2;
	}

	if( pMaskedData[ pMaskedData.length - 1 ] === 0 ) pMaskedData.length--;

	var strResult = "";

	var nMask = 0;

	var nChecksum = 0;
	len = pMaskedData.length;
	for( i=0; i<len; i++ ) {
		n = pMaskedData[i];
		var nLow = n % 256;
//		var nHigh = ( n - nLow ) / 256;
		var nHigh = ( n - nLow ) >> 8;
		var nMaskedCharCode = nLow * 256 + nHigh;

		nMask += 9973;
		nMask %= 65536;

		var nCharCode = ( nMaskedCharCode - nMask ) % 65536;
		if( nCharCode < 0 ) nCharCode += 65536;

		nChecksum += nCharCode;
		nChecksum %= 100;

		strResult += String.fromCharCode( nCharCode );
	}

	if( oPacked.checksum !== undefined && nChecksum !== oPacked.checksum ) {
		alert("ERROR: Checksum failed. Data corrupted.");
		return Bigint._error( "_packedDataToString", "Checksum failed: Recorded=" + oPacked.checksum + ": Observed=" + nChecksum );
	}

	strResult = strResult.replace( /\r\n/g , "\n" );

	return strResult;
}



PigPGP._writePig = function( pEncrypted, strSessionKeyRSAEncrypted, nChecksum ) {

	var strPig = "-----BEGIN PigPGP MESSAGE-----\n";
	strPig += "Version 0.2.4 http://www.faireal.net/demo/PigPGP/0_2_4\n\n";
	for( var i=0; i<pEncrypted.length; i++ ) {
		strPig += Bigint._toPigString( pEncrypted[i] );
		if( i % 10 === 9 ) strPig += "\n";
	}
	if( strPig.charAt( strPig.length - 1 ) !== "\n") strPig += "\n";

	var strFooter = "@v2:";
	if( nChecksum !== undefined ) strFooter += Bigint._toPigString( Bigint._padding2( nChecksum ) );
	strFooter += "@" + strSessionKeyRSAEncrypted;

	return strPig + PigPGP._formatKeyString( strFooter, "\n" ) + "-----END PigPGP MESSAGE-----\n\n";

}

PigPGP._readPig = function( strPig, oSecretKey ) {
	if( !strPig || strPig.indexOf("-----BEGIN PigPGP MESSAGE-----") === -1 )
		return Bigint._error("PigPGP._readPig", "header not found");
	strPig = strPig.replace( /\r\n?/g , "\n" );
	var bigUNIT = new Bigint( __BIGINT_2_32 );

	var pos1 = strPig.indexOf("\n\n") + 2;
	var pos2 = strPig.indexOf("@");
	var pos3 = strPig.lastIndexOf("@");
	var pos4 = strPig.lastIndexOf("\n-----");
	var strBody = strPig.substring( pos1, pos2 );

	var strInfo = strPig.substring( pos2+1, pos3 );
	strInfo = strInfo.replace( /\n/g , "" );
	var pos = strInfo.indexOf(":");

	var strSessionKeySystem = strInfo.substr( 0 , pos );

	if( strSessionKeySystem !== "v2" ) {
		return Bigint._error( "PigPGP._readPig", "Unknown Session Key System: " + strSessionKeySystem );
	}

	var strChecksum = strInfo.substr( pos + 1 );
	var nChecksum;
	if( strChecksum !== "" ) {
		nChecksum = Number( Bigint._pigStringToString( strChecksum ) );
	} else {
		nChecksum = undefined;
	}
	var strSessionKeyRSAEncrypted = strPig.substring( pos3+1, pos4 ).replace( /\s/g , "" );

	if( strBody === "" || strSessionKeyRSAEncrypted === "" )
		return Bigint._error("PigPGP._readPig", "parse error: Body or Key is null");

	var bigSessionKey = oSecretKey.decrypt( strSessionKeyRSAEncrypted );

	var bigQR = Bigint.divmod( bigSessionKey, bigUNIT );
	var b22 = bigQR[1].toNumber();
	bigQR = Bigint.divmod( bigQR[0], bigUNIT );
	var b21 = bigQR[1].toNumber();
	bigQR = Bigint.divmod( bigQR[0], bigUNIT );
	var b12 = bigQR[1].toNumber();
	var b11 = bigQR[0].toNumber();

	var nNewLineIndex;
	var pEncrypted = new Array();
	pos = 0;
	do {
		nNewLineIndex = strBody.indexOf("\n");
		var strPigLine = strBody.substring( 0, nNewLineIndex );

		var strNumber = Bigint._pigStringToString( strPigLine );
		var len = strNumber.length;
		for( var i=0; i<len; i+=10 ) {
			pEncrypted[pos++] = strNumber.substr( i, 10 )
		}
		strBody = strBody.substr( nNewLineIndex+1 );
	} while( strBody.length > 0 && nNewLineIndex !== -1 );

	var pDecrypted = PigPGP._unhill( pEncrypted, b11, b12, b21, b22 );

	var oPacked = new Object();
	oPacked.data = pDecrypted;
	oPacked.checksum = nChecksum;
	oPacked.version = strSessionKeySystem;

	return oPacked;
}




PigPGP._powmod_tunedF = function( bigBase, bigPower, bigModulus ) {
	var binaryString = bigPower.toBinaryString();
	var lastIndex = binaryString.length - 1;
	var bigWork = bigBase;
	var bigResult = ( binaryString.charAt( lastIndex ) === "1" )? bigBase.copy() : ( new Bigint(1) );

	_progress( "Decoding ( " + lastIndex + " steps ): " );

	for( var i = lastIndex - 1, cnt = 0; i > -1; i--, cnt++ ) {
		bigWork = Bigint._mod_typeF( Bigint._square_lite( bigWork ), bigModulus );
		if( binaryString.charAt(i) === "1" ) {
			bigResult = Bigint._mod_typeF( Bigint._mul_lite( bigResult, bigWork ), bigModulus );
		}
		if( cnt === 8 ) {
			_progress( Math.floor( ( lastIndex - i ) * 100 / lastIndex ) + "% " );
			cnt = 0;
		}
	}
	return bigResult;
}


PigPGP._powmod_tunedS = function( bigBase, bigPower, bigModulus ) {
	var binaryString = bigPower.toBinaryString();
	var lastIndex = binaryString.length - 1;
	var bigWork = bigBase;
	var bigResult = ( binaryString.charAt( lastIndex ) === "1" )? bigBase.copy() : ( new Bigint(1) );
	for( var i=lastIndex - 1; i > -1; i-- ) {
		bigWork = Bigint._mod_typeS( Bigint._square( bigWork ), bigModulus );
		if( binaryString.charAt(i) === "1" ) {
			bigResult = Bigint._mod_typeS( Bigint._mul( bigResult, bigWork ), bigModulus );
		}
		if( i % 32 === 0 ) _progress( 
			Math.floor( ( lastIndex - i ) * 100 / lastIndex ) + "% "
		);

	}
	return bigResult;
}


PigPGP._hill = function( pData, a11, a12, a21, a22 ) {
	if( pData.length & 0x1 ) pData[ pData.length ] = 0;

	var pResult = new Array();
	var len = pData.length;
	for( var i=0; i<len; i+=2 ) {
		var tmp11 = Bigint._mulmod32( a11, pData[ i ] );
		var tmp21 = Bigint._mulmod32( a21, pData[ i+1 ] );
		var nHigh = ( tmp11 + tmp21 ) >>> 0;

		var tmp12 = Bigint._mulmod32( a12, pData[ i ] );
		var tmp22 = Bigint._mulmod32( a22, pData[ i+1 ] );
		var nLow = ( tmp12 + tmp22 ) >>> 0;

		pResult[ i ] = Bigint._padding10( nHigh );
		pResult[ i+1 ] = Bigint._padding10( nLow );
	}
	return pResult;
}

PigPGP._unhill = function( pInput, b11, b12, b21, b22 ) {
	var pResult = new Array();
	var len = pInput.length;
	for( var i=0; i<len; i+=2 ) {
		var nHigh = pInput[i] - 0;
		var nLow = pInput[i+1] - 0;

		var tmp11 = Bigint._mulmod32( b11, nHigh );
		var tmp21 = Bigint._mulmod32( b21, nLow );
		pResult[i] = ( tmp11 + tmp21 ) >>> 0;

		var tmp12 = Bigint._mulmod32( b12, nHigh );
		var tmp22 = Bigint._mulmod32( b22, nLow );
		pResult[i+1] = ( tmp12 + tmp22 ) >>> 0;

	}
	if( pResult[ pResult.length - 1 ] === 0 ) pResult.length--;
	return pResult;
}


PigPGP.keygen = function( myForm ) {

	var bVerbose = myForm.VerboseOutput.checked;

	var fKeyLength = 128;
	if( myForm.KeyLength[1].checked ) fKeyLength = 256;
	else if( myForm.KeyLength[2].checked ) fKeyLength = 360;
	else if( myForm.KeyLength[3].checked ) fKeyLength = 512;
	else if( myForm.KeyLength[4].checked ) {
		fKeyLength = 1024;

	}

	var nSPSPDetection = 0;
	if( myForm.SPSPDetection[1].checked ) nSPSPDetection = 1;
	else if( myForm.SPSPDetection[2].checked ) nSPSPDetection = 2;
	else if( myForm.SPSPDetection[3].checked ) nSPSPDetection = 3;
	else if( myForm.SPSPDetection[4].checked ) nSPSPDetection = 4;

	if( fKeyLength > 200 ) {
		var message;

		if( __BIGINT_I18N && __BIGINT_I18N === "jpn" ) {
			message = fKeyLength + String.fromCharCode(12499, 12483, 12488, 12398, 37749, 12434, 29983, 25104, 12375, 12414, 12377, 12290, 12510, 12471, 12531, 12398, 36895, 24230, 12395, 12424, 12387, 12390, 25968, 21313, 31186, 12363, 12425, 25968, 20998, 12363, 12363, 12427, 22580, 21512, 12364, 12354, 12426, 12414, 12377, 12290 );
				//"ビットの鍵を生成します。マシンの速度によって数十秒から数分かかる場合があります。";
		} else {
			message = "PigPGP is about to start generating a " + fKeyLength + "-bit RSA key pair. ";
			message += "This may take 10 seconds to several minutes, depending on how fast your computer is.\n";
		}

		if( !confirm( message ) )
	 		return alert("Canceled.");
	} else if( fKeyLength < 128 ) fKeyLength = 128;
	PigPGP.init();

	var fKeyLengthDigit = fKeyLength * Math.LN2 / Math.LN10;
	var x = oMersenneTwister.getRandomUINT32Fluctuated() % 50000;
	var fBias = 0.4 + ( x / 1000000 ); // 0.40 ~ 0.45

	var nDigit = Math.floor( fKeyLengthDigit * fBias ) + 1;
	var bigStarter = oMersenneTwister.getRandomBigintOdd( nDigit );


	var bigP = PigPGP._getSPRP( bigStarter, nSPSPDetection, bVerbose );
	if( !bigP ) return false;

	if(bVerbose) Bigint._debug("P", bigP);

	var fBitP = bigP.getBitWidth(10);
	var fBitQ = fKeyLength - fBitP;

	var fBonusBit = ( oMersenneTwister.getRandomUINT32Fluctuated() % 30000000 ) / 100000000 + 0.1;

	var fQ = Math.floor( Math.pow( 2 , fBitQ + fBonusBit ) ); // make it 0.1 ~ 0.4 bit bigger than theory

	bigStarter = Bigint.fromFloat( fQ );
	bigStarter = oMersenneTwister._randomizeTail( bigStarter, 4 );
	var bigQ = PigPGP._getSPRP( bigStarter, nSPSPDetection, bVerbose );
	if( !bigQ ) return false;

	if( bVerbose ) Bigint._debug("Q", bigQ);

	var bigN = Bigint._mul_lite( bigP, bigQ );

	if( bVerbose ) Bigint._debug("N", bigN);

	var bigP_1 = Bigint._sub_typeN( bigP, 1 );
	var bigQ_1 = Bigint._sub_typeN( bigQ, 1 );
	var bigPhai = Bigint._mul_lite( bigP_1 , bigQ_1 );
	var strP_1 = bigP_1.toDecimalString();
	var strQ_1 = bigQ_1.toDecimalString();

	var nD_minDigits = ( bigN.getLength() / 4 ) + 2;

	var nE, bigE, bigD;
	for(var i = 3; i < 20; i++ ) {
		nE = __BIGINT_PRIMES[ i ];
		if( !Bigint._isDivisible_typeN( strP_1, nE ) && !Bigint._isDivisible_typeN( strQ_1, nE) ) {
			bigE = new Bigint( nE );
			bigD = Bigint._revmod( bigE, bigPhai );
			if( bigD.getLength() <= nD_minDigits ) {
				Bigint._debug( "bigD too small: " + bigD );
				continue;
			} else {
				break;
			}
		}
	}

	var pigN = bigN.toPigString();

	myForm.pubkey.value = PigPGP._formatKeyString( "PUB_PIGv2@" + bigE.toPigString() + "@" + pigN , "]" );
	myForm.seckey.value = PigPGP._formatKeyString( "SEC_PIGv2@" + bigD.toPigString() + "@" + pigN , "]"  );

	var fBit = bigN.getBitWidthS(2);
	Bigint._debug("Key Length" , fBit+"-bit");

	var nTotalCost = _timing("total");

	alert("Done. Cost: " + nTotalCost/1000 + " Seconds.");

	defaultStatus = "";

	return true;

}



PigPGP._getSPRP = function ( bigNumber, nSPSPDetection, bVerbose ) {
	var tableSmallPrimes = [ 2, 3, 5, 7, 11, 13, 17, 19 ];

	var big2 = new Bigint(2);



	if( !( bigNumber.table[0] & 1 ) ) bigNumber._increment(1);


	while( !bigNumber._is_3n() ) bigNumber._increment(2);

	// Starting from 3n, so you get 3(n+2m) if you add 6m, which can be skipped
	__BIGINT_GETSPRP_LOOP: for( var i=1; i<1000; i++ ) {
		bigNumber._increment( 2 );

		if( i % 3 === 0 || bigNumber._is_5n() ) continue;

		if( bigNumber._eratosthenes() ) {

			if(bVerbose) _timing( "Eratosthenes: " + bigNumber );
			_progress("-");
			continue;
		}

			_progress("*");

		if( bigNumber._isStrongProbablePrime( big2, bVerbose ) ) {
			if(bVerbose) _timing("Miller-Rabin");
			_progress("^");
			Bigint._debug( "SPRP(2)" );

		} else {
			if(bVerbose) _timing("Miller-Rabin");
			if(bVerbose) Bigint._debug( "NOT prime" );
			continue;
		}

		for( i = 0; i < nSPSPDetection; i++ ) {
			var bigBase = new Bigint( tableSmallPrimes[ i+1 ] );
			if( bigNumber._isStrongProbablePrime( bigBase,bVerbose ) ) {
				if(bVerbose) _timing("Miller-Rabin");
				_progress("^");
				Bigint._debug( "SPRP("+ tableSmallPrimes[ i+1 ] +")" );
			} else {
				if(bVerbose) _timing("Miller-Rabin");
				_progress("!")
				_debug( "### Pseudoprime(2) ###" , bigNumber );
				continue __BIGINT_GETSPRP_LOOP;
			}
		}

		return bigNumber;

	}
	
	return Bigint._error( "PigPGP._getSPRP", "Loops failed");

}

PigPGP.keyinfo = function( myForm ) {
	PigPGP.init();
	var strPublicKey = myForm.pubkey.value;
	var strSecretKey = myForm.seckey.value;
	if( !strPublicKey && !strSecretKey ) {
		alert( "Nothing in the Key fields." )
		return;
	}

	var nErrors = 0;

	if( strPublicKey ) {
		var pigPublicKey = PigPGP._parsePigKey( strPublicKey );
		if( pigPublicKey ) {
			myForm.debug.value += "* KEY INFO ABOUT:\n" + pigPublicKey.asciiString + "\n\n" + pigPublicKey + "\n\n";
		} else {
			myForm.output.value += "ERROR: THIS IS NOT A VALID PUBLIC KEY:\n\n" + strPublicKey + "\n\n";
			nErrors++;
		}

		myForm.debug.value += "-----\n\n";
	}

	if( strSecretKey ) {
		var pigSecretKey = PigPGP._parsePigKey( strSecretKey );
		if( pigSecretKey ) {
			myForm.debug.value += "* KEY INFO ABOUT:\n" + pigSecretKey.asciiString + "\n\n" + pigSecretKey + "\n\n";
		} else {
			myForm.debug.value += "ERROR: THIS IS NOT A VALID SECRET KEY:\n\n" + strSecretKey + "\n\n";
			nErrors++;
		}
	}

	var message = "Key Info: ";
	if( nErrors ) message += nErrors + " error(s)."
	else message += pigPublicKey.modulus.getLength() + "-digit. "

	message += "\n";
	message += "See the 'Debug' field.";
	alert( message );

}


PigPGP.encrypt = function(myForm) {


	PigPGP.init();
	_timing(0);
	var strKey = myForm.pubkey.value;
	if( !strKey ) {
		alert( "Nothing in the 'Public Key' field." )
		return false;
	}

	var pigPublicKey = PigPGP._parsePigKey( strKey );

	if( !pigPublicKey ) {
		alert("ERROR: " + strKey + " is not a valid key."); return false;
	} else if( pigPublicKey.header === __BIGINT_PIG_HEADER_SEC ) {
		alert("ERROR: A secret key is in the 'key' field. Use the public key."); return false;
	}

	var strInput = myForm.input.value;
	var oPacked = PigPGP._stringToPackedData( strInput );

	var unit = __BIGINT_2_32;
	var bigUnit = new Bigint( unit );

	var a11 = oMersenneTwister.getRandomUINT32Fluctuated();
	var a12 = oMersenneTwister.getRandomUINT32Fluctuated();
	var a21 = oMersenneTwister.getRandomUINT32Fluctuated();
	var a22 = oMersenneTwister.getRandomUINT32Fluctuated();

	if( a11 & 1 ) {
		while( !( a22 & 1 ) ) a22 = oMersenneTwister.getRandomUINT32Fluctuated();

		while( ( a12 & 1 ) && ( a21 & 1 ) ) {
			a12 = oMersenneTwister.getRandomUINT32Fluctuated();
			a21 = oMersenneTwister.getRandomUINT32Fluctuated();
		}
	} else {
		while( !(a12 & 1) ) {
			a12 = oMersenneTwister.getRandomUINT32Fluctuated();
		}

		while( !(a21 & 1) ) {
			a21 = oMersenneTwister.getRandomUINT32Fluctuated();
		}
	}

	do {
		a22 += 2;
		//a22 >>> 0;
		a22 %= unit;
		var bigDet = Bigint.sub( Bigint.mul(a11,a22) , Bigint.mul(a12,a21) );
		bigDet = Bigint.mod( bigDet, bigUnit );
		var bigGcd = Bigint._gcd( bigDet, bigUnit );
	} while( !bigGcd.isEqualTo( 1 ) );

	var rev = Bigint._revmod( bigDet, bigUnit ).toNumber();
	var bigB11 = new Bigint( Bigint._mulmod32(a22, rev) );
	var bigB12 = new Bigint( Bigint._mulmod32((unit - a12), rev) );
	var bigB21 = new Bigint( Bigint._mulmod32((unit - a21), rev) );
	var bigB22 = new Bigint( Bigint._mulmod32(a11, rev) );

//	var matrixA = [a11,a12,a21,a22];
//	var matrixB = [bigB11,bigB12,bigB21,bigB22];

	Bigint._debug( "Matrix" , bigB11.toNumber() + ", " + bigB12.toNumber() + ", " + bigB21.toNumber() + ", " + bigB22.toNumber() );


	var bigUnit2 = Bigint._square_lite( bigUnit );
	var bigUnit3 = Bigint._mul_lite( bigUnit2, bigUnit );
	var bigSessionKey = bigB22.copy();
	var bigWork = Bigint._mul_lite( bigB21 , bigUnit );
	bigSessionKey = Bigint._add( bigSessionKey, bigWork );
	bigWork = Bigint._mul_lite( bigB12, bigUnit2 );
	bigSessionKey = Bigint._add( bigSessionKey, bigWork );
	bigWork = Bigint._mul_lite( bigB11, bigUnit3 );
	bigSessionKey = Bigint._add( bigSessionKey, bigWork );


	var strSessionKeyRSAEncrypted = pigPublicKey.encrypt( bigSessionKey );

	var pEncrypted = PigPGP._hill( oPacked.data, a11, a12, a21, a22 );

	var strPig = PigPGP._writePig( pEncrypted, strSessionKeyRSAEncrypted, oPacked.checksum );

	myForm.input.value = strPig;
	myForm.output.value = "";

	defaultStatus = "";
	_timing("total");
	return true;
}



PigPGP.decrypt = function(myForm) {

	_timing(0);

	var strPig = myForm.input.value;
	if( !strPig ) return alert("Null input");

	strPig = strPig.replace(/\r\n?/g , "\n");

	PigPGP.init();
	var strKey = myForm.seckey.value;
	if( !strKey ) {
		alert( "Nothing in the 'Key' field." )
		return false;
	}

	var pigSecKey = PigPGP._parsePigKey( strKey );

	if( !pigSecKey ) {
		alert("ERROR: " + strKey + " is not a valid key."); return false;
	} else if(pigSecKey.header === __BIGINT_PIG_HEADER_PUB) {
		alert("ERROR: A public key is in the 'key' field. Use the secret key."); return false;
	}

	var oPacked = PigPGP._readPig( strPig, pigSecKey );

	myForm.input.value = PigPGP._packedDataToString( oPacked );
	myForm.output.value += " Done.";

	defaultStatus = "";
	_timing("total");

	return true;
}

PigPGP.init = function() {
	var myForm = document.__BIGINT_HTML_FORM;

	myForm.output.value = "";
	myForm.debug.value = "----- PigPGP 0.2.4 DEBUG LOG -----\n" + navigator.userAgent + "\n";
	defaultStatus = "";
	var d = new Date();
	var Y = d.getFullYear();
	var M = Bigint._padding2( d.getMonth() + 1 );
	var D = Bigint._padding2( d.getDate() );
	var h = Bigint._padding2( d.getHours() );
	var m = Bigint._padding2( d.getMinutes() );
	var s = Bigint._padding2( d.getSeconds() );
	var ms = d.getTime() % 1000;
	ms = Bigint._padding3( ms );
	var offs = d.getTimezoneOffset();
	var TZ_sign = ( offs > 0 )? "-" : "+" ;
	offs = Math.abs( offs );
	var TZ_m = Bigint._padding2( offs % 60 );
	var TZ_h = Bigint._padding2( ( offs - TZ_m ) / 60 );
	var TZ = TZ_sign + TZ_h + ":" + TZ_m;
	myForm.debug.value += "Current Time: "
		 +  Y + "-" + M + "-" + D + "T" + h + ":" + m + ":" + s + "." + ms + TZ + "\n";

	if(myForm.VerboseOutput.checked) {
		myForm.debug.value += "Verbose Mode: YES\n";
	}

	var flag = false;
	for( var i=0; i<myForm.KeyLength.length; i++) {
		if( myForm.KeyLength[i].checked ) {
			flag = true;
			break;
		}
	}
	if(!flag) myForm.KeyLength[0].checked = true;

	flag = false;
	for( i=0; i<myForm.SPSPDetection.length; i++) {
		if( myForm.SPSPDetection[i].checked ) {
			flag = true;
			break;
		}
	}
	if(!flag) myForm.SPSPDetection[0].checked = true;


	if( myForm.UnicodeSupportSelfTest.checked && !PigPGP.init.bUnicodeTested ) {
		var strUnicodeTest = String.fromCharCode( 12354, 12356, 12358, 12360, 12362 );
		if( myForm.UnicodeSupportChecker.value===strUnicodeTest ) 
			Bigint._debug( "Unicode Support Self-Test: Passed" );
		else alert( "Unicode Support Self-Test failed. You may not be able to use Unicode." );
		PigPGP.init.bUnicodeTested = true;
	}


	if( typeof oMersenneTwister === "undefined" ) {
		var nInitialSeed = Math.floor( Math.random() * 0xFFFFFFFF );
		oMersenneTwister = new MersenneTwister( nInitialSeed );
		nInitialSeed = oMersenneTwister.getRandomUINT32Fluctuated();
		oMersenneTwister = new MersenneTwister( nInitialSeed );
	}


	_timing(0);

}

PigPGP.init.bUnicodeTested = false;



/*
	OVERRIDE TO USE _square_lite
*/
Bigint.prototype._isStrongProbablePrime = function( bigBase, bVerbose ) {

	// NOTE: "this" MUST be odd (the number is 2 or non-prime if it is even )

	var big1 = new Bigint( 1 );
	var bigMinus1 = Bigint._sub_typeN( this , 1 );
	var bigA = bigMinus1.copy();

	for( var nMaxSteps = 0; bigA.isEven(); nMaxSteps++ ) {
		bigA.half();
	}


	if( bVerbose ) Bigint._debug( "Checking " + this );

	var bigRemainder = Bigint._powmod( bigBase, bigA, this );

	if( bigRemainder._isEqualTo( big1 ) || bigRemainder._isEqualTo( bigMinus1 ) ) {
		Bigint._debug("* Passed at Step 1 / " +  nMaxSteps );
		return true;
	}
	for( var nSteps = 1; nSteps < nMaxSteps; nSteps++ ) {

		bigRemainder = Bigint._mod_typeF( Bigint._square_lite( bigRemainder ), this );


		if( bigRemainder._isEqualTo( bigMinus1 ) ) {
			Bigint._debug("* Passed at Step " + ( 1 + nSteps ) + " / " + nMaxSteps );
			return true;
		}

	}
	return false;
}


function _progress( str ) {
		var p = document.__BIGINT_HTML_FORM.output;
		if( str === undefined ) p.value = "";
		else {
			p.value += str;
			if( p.value.length % 80 === 79 ) p.value += "\n";
			if( !__BIGINT_MOZ ) {
				var pos = p.value.lastIndexOf("\n");
				defaultStatus = p.value.substr( pos + 1 );
			}
		}
}
