// sub.js
//////////////////////////////////////////////////////////////////////////////////////

Bigint.sub = function( number1, number2 ) {
	var bigNumber1 = Bigint.serio( number1 );
	var bigNumber2 = Bigint.serio( number2 );
	var bigResult, cmp;

	if( bigNumber1.isNonNegative ) {
		if( bigNumber2.isNonNegative ) {
			cmp = Bigint._tablecmp( bigNumber1.table, bigNumber2.table );
			if( cmp > 0 ) {
				bigResult = Bigint._sub( bigNumber1, bigNumber2 );
			} else if( cmp === 0 ) {
				return new Bigint(0);
			} else {
				bigResult = Bigint._sub( bigNumber2, bigNumber1 );
				bigResult.isNonNegative = false;
			}
		} else {
			bigResult = Bigint._add( bigNumber1, bigNumber2 );
		}
	} else { // bigNumber1 is negative
		if( bigNumber2.isNonNegative ) {
			bigResult = Bigint._add( bigNumber1, bigNumber2 );
			bigResult.isNonNegative = false;
		} else {
			cmp = Bigint._tablecmp( bigNumber1.table, bigNumber2.table );
			if( cmp > 0 ) {
				bigResult = Bigint._sub( bigNumber1, bigNumber2 );
				bigResult.isNonNegative = false;
			} else if( cmp === 0 ) {
				return new Bigint(0);
			} else {
				bigResult = Bigint._sub( bigNumber2, bigNumber1 );
			}
		}
	}
	return bigResult;
}

Bigint._sub = function( bigNumber1, bigNumber2 ) { // | n1 | - | n2 | and | n1 | is always greater
	var table1 = bigNumber1.table, table2 = bigNumber2.table;
	var nLength1 = table1.length, nLength2 = table2.length;

	if( nLength1 === nLength2 ) {
		var nCounter = 0;
		for( var i = nLength1 - 1; table1[ i ] === table2[ i ] && i > -1; i-- ) nCounter++;
		nLength1 -= nCounter;
		if( nLength1 === 0 ) return new Bigint( 0 );
		nLength2 = nLength1;
	}

	var tableResult = new Array( nLength1 );

	var flag = 0;
	for( i = 0; i < nLength2; i++ ) {
		tableResult[ i ] = table1[ i ] - ( table2[ i ] + flag );
		if( tableResult[ i ] < 0 ) {
			tableResult[ i ]  += __BIGINT_UNIT;
			flag = 1;
		} else {
			flag = 0;
		}
	}

	if( flag ) {
		for( ; table1[ i ] === 0; i++ ) tableResult[ i ] = __BIGINT_UNIT_MINUS_1;

		if( table1[ i ] === undefined ) Bigint._error( "_sub" , "Probably the 2nd number is bigger than the 1st number" );

		tableResult[ i ] = table1[ i ] - 1;
		i++;
	}
	for( ; i < nLength1; i++ ) tableResult[ i ] = table1[ i ];

	var nCurrentTableLength = tableResult.length;
	var nVacantCells = 0;
	for( i = nCurrentTableLength - 1, nVacantCells = 0; !tableResult[ i ] && i > 0 ; i-- ) { nVacantCells++; }
	if( nVacantCells > 0 ) tableResult.length = nCurrentTableLength - nVacantCells;

	var bigResult = new Bigint();
	bigResult.table = tableResult;
	return bigResult;
}

Bigint._sub_typeN = function( bigNumber, smallNumber ) { // smallNumber must be < __BIGINT_UNIT
	var tableInput = bigNumber.table;
	var nLength = tableInput.length;
//	if( nLength === 1 ) return new Bigint( tableInput[ 0 ] - smallNumber );

	var tableResult = new Array( nLength );

	tableResult[ 0 ] = tableInput[ 0 ] - smallNumber;
	var idx = 1;

	if( tableResult[ 0 ] < 0 ) {
		tableResult[ 0 ] += __BIGINT_UNIT;
		for( ; tableInput[ idx ] === 0; idx++ ) { tableResult[ idx ] = __BIGINT_UNIT_MINUS_1; }
		tableResult[ idx ] = tableInput[ idx ] - 1;
		idx++;
	}

	for( ; idx < nLength; idx++ ) tableResult[ idx ] = tableInput[ idx ];

	if( tableResult[ idx - 1 ] === 0 && idx > 0 ) tableResult.length--;

	var bigResult = new Bigint();
	bigResult.table = tableResult;
	return bigResult;
}


/*
Bigint._sub_typeN = function( bigNumber, smallNumber ) {
	var bigResult = bigNumber.copy();
	var flag = 0;
	bigResult.table[0] -= smallNumber;
	
	if( bigResult.table[0] >= 0 ) {
		return bigResult;
	} else {
		bigResult.table[0] += __BIGINT_BIG_UNIT;
		flag = 1;
	}

	for( var i=1; i<bigResult.table.length; i++ ) {
		bigResult.table[i] -= flag;
		if( bigResult.table[i] < 0 ) {
			bigResult.table[i] += __BIGINT_BIG_UNIT;
			flag = 1;
		} else {
			break;
		}
	}

	var lastIndex = bigResult.table.length - 1;
	if( bigResult.table[ lastIndex ] === 0 ) bigResult.table.length--;

	return bigResult;
}
*/

Bigint.prototype.decrement = function( input ) {
	if( input === undefined ) {
		this._decrement( 1 );
	} else if( input.isBigint || input >= __BIGINT_UNIT || input < 0 || !this.isNonNegative ) {
		var bigResult = Bigint.sub( this , input );
		this.table = bigResult.table;
		this.isNonNegative = bigResult.isNonNegative;
	} else {
		var n = Math.floor( input );
		if( isNaN( n ) ) {
			Bigint._error( "decrement" , "Bad input: " + input );
		} else {
			this._decrement( n );
		}
	}
}

Bigint.prototype._decrement = function( n ) {
	if( n === undefined ) n = 1;
	var table = this.table;
	if( table.length > 1 ) {
		table[ 0 ] -= n;
		if( table[ 0 ] < 0 ) {
			table[ 0 ] += __BIGINT_UNIT;
			for( var i = 1; table[ i ] === 0; i++ ) table[ i ] = __BIGINT_UNIT_MINUS_1;
			table[ i ]--;
			if( table[ i ] === 0 ) table.length--;
		}
	} else { // this.table.length === 1
		if( table[ 0 ] >= n ) {
			table[ 0 ] -= n;
		} else {
			table[ 0 ] = n - table[ 0 ];
			this.isNonNegative = false;
		}
	}
}


Bigint._strsub = function ( strNumber1, strNumber2 ) {

	var pos1 = strNumber1.length;
	var pos2 = strNumber2.length;

	if( pos1 < pos2 ) return undefined;

	var biggerTable = new Array();

	var n1, n2;
	var flag = 0;
	var i = 0;
	for( ; pos2 > 0; i++, pos1 -= 15, pos2 -= 15 ) {
		n1 = Number( strNumber1.substring( pos1 - 15, pos1 ) );
		n2 = Number( strNumber2.substring( pos2 - 15, pos2 ) );
		biggerTable[ i ] = n1 - ( n2 + flag );



		if( biggerTable[ i ] >= 0 ) {
			flag = 0;
		} else {
			biggerTable[ i ] += __BIGINT_BIGGER_UNIT;
			flag = 1;
		}


	}

	if( flag ) {
		for( ; pos1 > 0; i++ ) {
			n1 = Number( strNumber1.substring( pos1 - 15, pos1 ) );
			pos1 -= 15;
			if( n1 > 0 ) {
				biggerTable[ i ] = n1 - 1;
				flag = 0;
				break;
			} else {
				biggerTable[ i ] = __BIGINT_BIGGER_UNIT - 1;
			}
		}
		if( flag ) {
			return undefined;
		}
	}



	var nLastIndex = biggerTable.length - 1;


	var strResult;
	if( pos1 > 0 ) {
		strResult = strNumber1.substring( 0 , pos1 ) + Bigint._paddingBigger( biggerTable[ nLastIndex ] );
	} else {
		while( !biggerTable[ nLastIndex ] ) {
			nLastIndex--;
			if( nLastIndex < 0 ) return "";
		}
		strResult = String( biggerTable[ nLastIndex ] );
	}

	for( i = nLastIndex - 1; i >= 0; i-- ) {
		strResult += Bigint._paddingBigger( biggerTable[ i ] );
	}

	return strResult;
}


